#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Database v1 Users action implementations"""

from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils

from troveclient.i18n import _


class CreateDatabaseUser(command.Command):

    _description = _("Creates a user on an instance.")

    def get_parser(self, prog_name):
        parser = super(CreateDatabaseUser, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or name of the instance.')
        )
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of user.')
        )
        parser.add_argument(
            'password',
            metavar='<password>',
            help=_('Password of user.')
        )
        parser.add_argument(
            '--host',
            metavar='<host>',
            help=_('Optional host of user.')
        )
        parser.add_argument(
            '--databases',
            metavar='<databases>',
            nargs='+',
            default=[],
            help=_('Optional list of databases.')
        )
        return parser

    def take_action(self, parsed_args):
        manager = self.app.client_manager.database
        users = manager.users
        instance = utils.find_resource(manager.instances,
                                       parsed_args.instance)
        databases = [{'name': value} for value in parsed_args.databases]
        user = {'name': parsed_args.name, 'password': parsed_args.password,
                'databases': databases}
        if parsed_args.host:
            user['host'] = parsed_args.host
        users.create(instance, [user])


class ListDatabaseUsers(command.Lister):

    _description = _("Lists the users for an instance.")
    columns = ['Name', 'Host', 'Databases']

    def get_parser(self, prog_name):
        parser = super(ListDatabaseUsers, self).get_parser(prog_name)
        parser.add_argument(
            dest='instance',
            metavar='<instance>',
            help=_('ID or name of the instance.')
        )
        return parser

    def take_action(self, parsed_args):
        manager = self.app.client_manager.database
        db_users = manager.users
        instance = utils.find_resource(manager.instances,
                                       parsed_args.instance)
        items = db_users.list(instance)
        users = items
        while (items.next):
            items = db_users.list(parsed_args.instance, marker=items.next)
            users += items
        for user in users:
            db_names = [db['name'] for db in user.databases]
            user.databases = ', '.join(db_names)
        users = [utils.get_item_properties(u, self.columns) for u in users]
        return self.columns, users


class ShowDatabaseUser(command.ShowOne):

    _description = _("Shows details of a database user of an instance.")

    def get_parser(self, prog_name):
        parser = super(ShowDatabaseUser, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or name of the instance.'),
        )
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of user.'),
        )
        parser.add_argument(
            "--host",
            metavar="<host>",
            help=_("Optional host of user."),
        )
        return parser

    def take_action(self, parsed_args):
        manager = self.app.client_manager.database
        db_users = manager.users
        instance = utils.find_resource(manager.instances,
                                       parsed_args.instance)
        user = db_users.get(instance, parsed_args.name,
                            hostname=parsed_args.host)
        return zip(*sorted(user._info.items()))


class DeleteDatabaseUser(command.Command):

    _description = _("Deletes a user from an instance.")

    def get_parser(self, prog_name):
        parser = super(DeleteDatabaseUser, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or name of the instance.')
        )
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of user.')
        )
        parser.add_argument(
            '--host',
            metavar='<host>',
            help=_('Optional host of user.')
        )
        return parser

    def take_action(self, parsed_args):
        manager = self.app.client_manager.database
        users = manager.users
        try:
            instance = utils.find_resource(manager.instances,
                                           parsed_args.instance)
            users.delete(instance, parsed_args.name, parsed_args.host)
        except Exception as e:
            msg = (_("Failed to delete user %(user)s: %(e)s")
                   % {'user': parsed_args.name, 'e': e})
            raise exceptions.CommandError(msg)


class GrantDatabaseUserAccess(command.Command):

    _description = _("Grants access to a database(s) for a user.")

    def get_parser(self, prog_name):
        parser = super(GrantDatabaseUserAccess, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or name of the instance.')
        )
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of user.')
        )
        parser.add_argument(
            '--host',
            metavar='<host>',
            default=None,
            help=_('Optional host of user.')
        )
        parser.add_argument(
            'databases',
            metavar='<databases>',
            nargs="+",
            default=[],
            help=_('List of databases.')
        )
        return parser

    def take_action(self, parsed_args):
        manager = self.app.client_manager.database
        users = manager.users
        instance = utils.find_resource(manager.instances,
                                       parsed_args.instance)
        users.grant(instance, parsed_args.name,
                    parsed_args.databases, hostname=parsed_args.host)


class RevokeDatabaseUserAccess(command.Command):

    _description = _("Revokes access to a database for a user.")

    def get_parser(self, prog_name):
        parser = super(RevokeDatabaseUserAccess, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or name of the instance.')
        )
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of user.')
        )
        parser.add_argument(
            '--host',
            metavar='<host>',
            default=None,
            help=_('Optional host of user.')
        )
        parser.add_argument(
            'databases',
            metavar='<databases>',
            help=_('A single database.')
        )
        return parser

    def take_action(self, parsed_args):
        manager = self.app.client_manager.database
        users = manager.users
        instance = utils.find_resource(manager.instances,
                                       parsed_args.instance)
        users.revoke(instance, parsed_args.name,
                     parsed_args.databases, hostname=parsed_args.host)


class ShowDatabaseUserAccess(command.Lister):

    _description = _("Shows access details of a user of an instance.")
    columns = ['Name']

    def get_parser(self, prog_name):
        parser = super(ShowDatabaseUserAccess, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or name of the instance.')
        )
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of user.')
        )
        parser.add_argument(
            '--host',
            metavar='<host>',
            default=None,
            help=_('Optional host of user.')
        )
        return parser

    def take_action(self, parsed_args):
        manager = self.app.client_manager.database
        users = manager.users
        instance = utils.find_resource(manager.instances,
                                       parsed_args.instance)
        names = users.list_access(instance, parsed_args.name,
                                  hostname=parsed_args.host)
        access = [utils.get_item_properties(n, self.columns) for n in names]
        return self.columns, access


class UpdateDatabaseUserAttributes(command.Command):

    _description = _("Updates a user's attributes on an instance."
                     "At least one optional argument must be provided.")

    def get_parser(self, prog_name):
        parser = super(UpdateDatabaseUserAttributes,
                       self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or name of the instance.')
        )
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of user.')
        )
        parser.add_argument(
            '--host',
            metavar='<host>',
            default=None,
            help=_('Optional host of user.')
        )
        parser.add_argument(
            '--new_name',
            metavar='<new_name>',
            default=None,
            help=_('Optional new name of user.')
        )
        parser.add_argument(
            '--new_password',
            metavar='<new_password>',
            default=None,
            help=_('Optional new password of user.')
        )
        parser.add_argument(
            '--new_host',
            metavar='<new_host>',
            default=None,
            help=_('Optional new host of user.')
        )
        return parser

    def take_action(self, parsed_args):
        manager = self.app.client_manager.database
        users = manager.users
        instance = utils.find_resource(manager.instances,
                                       parsed_args.instance)
        new_attrs = {}
        if parsed_args.new_name:
            new_attrs['name'] = parsed_args.new_name
        if parsed_args.new_password:
            new_attrs['password'] = parsed_args.new_password
        if parsed_args.new_host:
            new_attrs['host'] = parsed_args.new_host
        users.update_attributes(instance, parsed_args.name,
                                newuserattr=new_attrs,
                                hostname=parsed_args.host)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

from unittest import mock

from osc_lib import utils
from oslo_utils import uuidutils

from troveclient import common
from troveclient import exceptions
from troveclient.osc.v1 import database_instances
from troveclient.tests.osc.v1 import fakes
from troveclient.v1 import instances


class TestInstances(fakes.TestDatabasev1):
    def setUp(self):
        super(TestInstances, self).setUp()

        self.fake_instances = fakes.FakeInstances()
        self.instance_client = self.app.client_manager.database.instances
        self.mgmt_client = self.app.client_manager.database.mgmt_instances


class TestInstanceList(TestInstances):
    defaults = {
        'include_clustered': False,
        'limit': None,
        'marker': None
    }

    def setUp(self):
        super(TestInstanceList, self).setUp()
        self.cmd = database_instances.ListDatabaseInstances(self.app, None)

    def test_instance_list_defaults(self):
        instance_id = self.random_uuid()
        name = self.random_name('test-list')
        tenant_id = self.random_uuid()
        insts = [
            {
                "id": instance_id,
                "name": name,
                "status": "ACTIVE",
                "operating_status": "HEALTHY",
                "addresses": [
                    {"type": "private", "address": "10.0.0.13"}
                ],
                "volume": {"size": 2},
                "flavor": {"id": "02"},
                "region": "regionOne",
                "datastore": {
                    "version": "5.6", "type": "mysql",
                    "version_number": "5.7.29"
                },
                "tenant_id": tenant_id,
                "replica_of": self.random_uuid(),
                "access": {"is_public": False, "allowed_cidrs": []},
            }
        ]
        self.instance_client.list.return_value = common.Paginated(
            [instances.Instance(mock.MagicMock(), inst) for inst in insts])

        parsed_args = self.check_parser(self.cmd, [], [])
        columns, data = self.cmd.take_action(parsed_args)

        self.instance_client.list.assert_called_once_with(**self.defaults)
        self.assertEqual(
            database_instances.ListDatabaseInstances.columns,
            columns
        )

        values = [
            (instance_id, name, 'mysql', '5.6', 'ACTIVE', 'HEALTHY', False,
             [{"type": "private", "address": "10.0.0.13"}],
             '02', 2, 'replica'),
        ]
        self.assertEqual(values, data)

    def test_instance_list_all_projects(self):
        instance_id = self.random_uuid()
        name = self.random_name('test-list')
        tenant_id = self.random_uuid()
        server_id = self.random_uuid()
        insts = [
            {
                "id": instance_id,
                "name": name,
                "status": "ACTIVE",
                "operating_status": "HEALTHY",
                "addresses": [
                    {"type": "private", "address": "10.0.0.13"}
                ],
                "volume": {"size": 2},
                "flavor": {"id": "02"},
                "region": "regionOne",
                "datastore": {
                    "version": "5.6", "type": "mysql",
                    "version_number": "5.7.29"
                },
                "tenant_id": tenant_id,
                "access": {"is_public": False, "allowed_cidrs": []},
                "server_id": server_id,
                'server': {
                    'id': server_id
                }
            }
        ]
        self.mgmt_client.list.return_value = common.Paginated(
            [instances.Instance(mock.MagicMock(), inst) for inst in insts])

        parsed_args = self.check_parser(self.cmd, ["--all-projects"],
                                        [("all_projects", True)])
        columns, data = self.cmd.take_action(parsed_args)

        self.mgmt_client.list.assert_called_once_with(**self.defaults)
        self.assertEqual(
            database_instances.ListDatabaseInstances.admin_columns,
            columns
        )

        expected_instances = [
            (instance_id, name, 'mysql', '5.6', 'ACTIVE', 'HEALTHY', False,
             [{"type": "private", "address": "10.0.0.13"}],
             '02', 2, '', server_id, tenant_id),
        ]
        self.assertEqual(expected_instances, data)

    def test_instance_list_for_project(self):
        instance_id = self.random_uuid()
        name = self.random_name('test-list')
        tenant_id = self.random_uuid()
        server_id = self.random_uuid()
        insts = [
            {
                "id": instance_id,
                "name": name,
                "status": "ACTIVE",
                "operating_status": "HEALTHY",
                "addresses": [
                    {"type": "private", "address": "10.0.0.13"}
                ],
                "volume": {"size": 2},
                "flavor": {"id": "02"},
                "region": "regionOne",
                "datastore": {
                    "version": "5.6", "type": "mysql",
                    "version_number": "5.7.29"
                },
                "tenant_id": tenant_id,
                "access": {"is_public": False, "allowed_cidrs": []},
                "server_id": server_id,
                'server': {
                    'id': server_id
                }
            }
        ]
        self.mgmt_client.list.return_value = common.Paginated(
            [instances.Instance(mock.MagicMock(), inst) for inst in insts])

        parsed_args = self.check_parser(self.cmd, ["--project-id", tenant_id],
                                        [("project_id", tenant_id)])
        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(
            database_instances.ListDatabaseInstances.admin_columns,
            columns
        )
        expected_instances = [
            (instance_id, name, 'mysql', '5.6', 'ACTIVE', 'HEALTHY', False,
             [{"type": "private", "address": "10.0.0.13"}],
             '02', 2, '', server_id, tenant_id),
        ]
        self.assertEqual(expected_instances, data)

        expected_params = {
            'include_clustered': False,
            'limit': None,
            'marker': None,
            'project_id': tenant_id
        }
        self.mgmt_client.list.assert_called_once_with(**expected_params)


class TestInstanceShow(TestInstances):
    def setUp(self):
        super(TestInstanceShow, self).setUp()
        self.cmd = database_instances.ShowDatabaseInstance(self.app, None)
        self.columns = (
            'addresses',
            'allowed_cidrs',
            'datastore',
            'datastore_version',
            'datastore_version_number',
            'flavor',
            'id',
            'name',
            'operating_status',
            'public',
            'region',
            'replica_of',
            'status',
            'tenant_id',
            'volume',
        )

    def test_show(self):
        instance_id = self.random_uuid()
        name = self.random_name('test-show')
        flavor_id = self.random_uuid()
        primary_id = self.random_uuid()
        tenant_id = self.random_uuid()
        inst = {
            "id": instance_id,
            "name": name,
            "status": "ACTIVE",
            "operating_status": "HEALTHY",
            "addresses": [
                {"type": "private", "address": "10.0.0.13"}
            ],
            "volume": {"size": 2},
            "flavor": {"id": flavor_id},
            "region": "regionOne",
            "datastore": {
                "version": "5.7.29", "type": "mysql",
                "version_number": "5.7.29"
            },
            "tenant_id": tenant_id,
            "replica_of": {'id': primary_id},
            "access": {"is_public": False, "allowed_cidrs": []},
        }
        self.instance_client.get.return_value = instances.Instance(
            mock.MagicMock(), inst)

        parsed_args = self.check_parser(self.cmd, [instance_id], [])
        columns, data = self.cmd.take_action(parsed_args)

        values = ([{'address': '10.0.0.13', 'type': 'private'}], [], 'mysql',
                  '5.7.29', '5.7.29', flavor_id, instance_id, name, 'HEALTHY',
                  False, 'regionOne', primary_id, 'ACTIVE',
                  tenant_id, 2)
        self.assertEqual(self.columns, columns)
        self.assertEqual(values, data)


class TestDatabaseInstanceDelete(TestInstances):

    def setUp(self):
        super(TestDatabaseInstanceDelete, self).setUp()
        self.cmd = database_instances.DeleteDatabaseInstance(self.app, None)

    @mock.patch("troveclient.utils.get_resource_id_by_name")
    def test_instance_delete(self, mock_getid):
        mock_getid.return_value = "fake_uuid"

        args = ['instance1']
        parsed_args = self.check_parser(self.cmd, args, [])
        self.cmd.take_action(parsed_args)

        mock_getid.assert_called_once_with(self.instance_client, "instance1")
        self.instance_client.delete.assert_called_with('fake_uuid')

    @mock.patch("troveclient.utils.get_resource_id_by_name")
    def test_instance_delete_with_exception(self, mock_getid):
        mock_getid.side_effect = exceptions.CommandError

        args = ['fakeinstance']
        parsed_args = self.check_parser(self.cmd, args, [])

        self.assertRaises(exceptions.CommandError,
                          self.cmd.take_action,
                          parsed_args)

    @mock.patch("troveclient.utils.get_resource_id_by_name")
    def test_instance_bulk_delete(self, mock_getid):
        instance_1 = uuidutils.generate_uuid()
        instance_2 = uuidutils.generate_uuid()
        mock_getid.return_value = instance_1

        args = ["fake_instance", instance_2]
        parsed_args = self.check_parser(self.cmd, args, [])
        self.cmd.take_action(parsed_args)

        mock_getid.assert_called_once_with(self.instance_client,
                                           "fake_instance")

        calls = [mock.call(instance_1), mock.call(instance_2)]
        self.instance_client.delete.assert_has_calls(calls)

    @mock.patch("troveclient.utils.get_resource_id_by_name")
    def test_instance_force_delete(self, mock_getid):
        mock_getid.return_value = "fake_uuid"

        args = ['instance1', '--force']
        parsed_args = self.check_parser(self.cmd, args, [('force', True)])
        self.cmd.take_action(parsed_args)

        mock_getid.assert_called_once_with(self.instance_client, "instance1")
        self.instance_client.force_delete.assert_called_with('fake_uuid')


class TestDatabaseInstanceCreate(TestInstances):

    values = ('2017-12-22T20:02:32', 'mysql', '5.6', '5.7.29', '310',
              '2468', 'test', 'test-net', 'net-id', 'BUILD',
              '2017-12-22T20:02:32', 1)
    columns = (
        'created',
        'datastore',
        'datastore_version',
        'datastore_version_number',
        'flavor',
        'id',
        'name',
        'networks',
        'networks_id',
        'status',
        'updated',
        'volume',
    )

    def setUp(self):
        super(TestDatabaseInstanceCreate, self).setUp()
        self.cmd = database_instances.CreateDatabaseInstance(self.app, None)
        self.data = self.fake_instances.get_instance_create()
        self.instance_client.create.return_value = self.data

    @mock.patch.object(utils, 'find_resource')
    def test_instance_create(self, mock_find):
        args = ['test-name', '--flavor', '103',
                '--size', '1',
                '--databases', 'db1', 'db2',
                '--users', 'u1:111', 'u2:111',
                '--datastore', "datastore",
                '--datastore-version', "datastore_version",
                '--nic', 'net-id=net1',
                '--replica-of', 'test',
                '--replica-count', '4',
                '--module', 'mod_id',
                '--is-public',
                '--allowed-cidr', '10.0.0.1/24',
                '--allowed-cidr', '192.168.0.1/24']
        verifylist = [
            ('name', 'test-name'),
            ('flavor', '103'),
            ('size', 1),
            ('databases', ['db1', 'db2']),
            ('users', ['u1:111', 'u2:111']),
            ('datastore', "datastore"),
            ('datastore_version', "datastore_version"),
            ('nics', 'net-id=net1'),
            ('replica_of', 'test'),
            ('replica_count', 4),
            ('modules', ['mod_id']),
            ('is_public', True),
            ('allowed_cidrs', ['10.0.0.1/24', '192.168.0.1/24'])
        ]
        parsed_args = self.check_parser(self.cmd, args, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.values, data)

    @mock.patch.object(utils, 'find_resource')
    def test_instance_create_without_allowed_cidrs(self, mock_find):
        resp = {
            "id": "a1fea1cf-18ad-48ab-bdfd-fce99a4b834e",
            "name": "test-mysql",
            "status": "BUILD",
            "flavor": {
                "id": "a48ea749-7ee3-4003-8aae-eb4e79773e2d"
            },
            "datastore": {
                "type": "mysql",
                "version": "5.7.29",
                "version_number": "5.7.29"
            },
            "region": "RegionOne",
            "access": {
                "is_public": True
            },
            "volume": {
                "size": 1
            },
            "created": "2020-08-12T09:41:47",
            "updated": "2020-08-12T09:41:47",
            "service_status_updated": "2020-08-12T09:41:47"
        }
        self.instance_client.create.return_value = instances.Instance(
            mock.MagicMock(), resp)

        args = [
            'test-mysql',
            '--flavor', 'a48ea749-7ee3-4003-8aae-eb4e79773e2d',
            '--size', '1',
            '--datastore', "mysql",
            '--datastore-version', "5.7.29",
            '--nic', 'net-id=net1',
            '--is-public'
        ]
        verifylist = [
            ('name', 'test-mysql'),
            ('flavor', 'a48ea749-7ee3-4003-8aae-eb4e79773e2d'),
            ('size', 1),
            ('datastore', "mysql"),
            ('datastore_version', "5.7.29"),
            ('nics', 'net-id=net1'),
            ('is_public', True),
            ('allowed_cidrs', None)
        ]

        parsed_args = self.check_parser(self.cmd, args, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        expected_columns = (
            'allowed_cidrs',
            'created',
            'datastore',
            'datastore_version',
            'datastore_version_number',
            'flavor',
            'id',
            'name',
            'public',
            'region',
            'service_status_updated',
            'status',
            'updated',
            'volume',
        )
        expected_values = (
            [],
            "2020-08-12T09:41:47",
            "mysql",
            "5.7.29",
            "5.7.29",
            "a48ea749-7ee3-4003-8aae-eb4e79773e2d",
            "a1fea1cf-18ad-48ab-bdfd-fce99a4b834e",
            "test-mysql",
            True,
            "RegionOne",
            "2020-08-12T09:41:47",
            "BUILD",
            "2020-08-12T09:41:47",
            1,
        )
        self.assertEqual(expected_columns, columns)
        self.assertEqual(expected_values, data)

    @mock.patch.object(utils, 'find_resource')
    def test_instance_create_nic_param(self, mock_find):
        fake_id = self.random_uuid()
        mock_find.return_value.id = fake_id
        args = [
            'test-mysql',
            '--flavor', 'a48ea749-7ee3-4003-8aae-eb4e79773e2d',
            '--size', '1',
            '--datastore', "mysql",
            '--datastore-version', "5.7.29",
            '--nic', 'net-id=net1,subnet-id=subnet_id,ip-address=192.168.1.11',
        ]
        parsed_args = self.check_parser(self.cmd, args, [])
        self.cmd.take_action(parsed_args)

        self.instance_client.create.assert_called_once_with(
            'test-mysql',
            flavor_id=fake_id,
            volume={"size": 1, "type": None},
            databases=[],
            users=[],
            restorePoint=None,
            availability_zone=None,
            datastore='mysql',
            datastore_version='5.7.29',
            datastore_version_number=None,
            nics=[
                {'network_id': 'net1', 'subnet_id': 'subnet_id',
                 'ip_address': '192.168.1.11'}
            ],
            configuration=None,
            replica_of=None,
            replica_count=None,
            modules=[],
            locality=None,
            region_name=None,
            access={'is_public': False}
        )


class TestDatabaseInstanceResetStatus(TestInstances):

    def setUp(self):
        super(TestDatabaseInstanceResetStatus, self).setUp()
        self.cmd = database_instances.ResetDatabaseInstanceStatus(self.app,
                                                                  None)

    @mock.patch.object(utils, 'find_resource')
    def test_instance_reset_status(self, mock_find):
        args = ['instance1']
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])
        result = self.cmd.take_action(parsed_args)
        self.instance_client.reset_status.assert_called_with('instance1')
        self.assertIsNone(result)


class TestDatabaseInstanceResizeFlavor(TestInstances):

    def setUp(self):
        super(TestDatabaseInstanceResizeFlavor, self).setUp()
        self.cmd = database_instances.ResizeDatabaseInstanceFlavor(self.app,
                                                                   None)

    @mock.patch.object(utils, 'find_resource')
    def test_instance_resize_flavor(self, mock_find):
        args = ['instance1', 'flavor_id']
        mock_find.side_effect = [
            mock.MagicMock(id='fake_instance_id'),
            mock.MagicMock(id='fake_flavor_id')
        ]
        parsed_args = self.check_parser(self.cmd, args, [])
        self.cmd.take_action(parsed_args)

        self.instance_client.resize_instance.assert_called_with(
            'fake_instance_id', 'fake_flavor_id')


class TestDatabaseInstanceUpgrade(TestInstances):

    def setUp(self):
        super(TestDatabaseInstanceUpgrade, self).setUp()
        self.cmd = database_instances.UpgradeDatabaseInstance(self.app, None)

    @mock.patch.object(utils, 'find_resource')
    def test_instance_upgrade(self, mock_find):
        args = ['instance1', 'datastore_version1']
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])
        result = self.cmd.take_action(parsed_args)
        self.instance_client.upgrade.assert_called_with('instance1',
                                                        'datastore_version1')
        self.assertIsNone(result)


class TestDatabaseInstanceResizeVolume(TestInstances):

    def setUp(self):
        super(TestDatabaseInstanceResizeVolume, self).setUp()
        self.cmd = database_instances.ResizeDatabaseInstanceVolume(self.app,
                                                                   None)

    @mock.patch.object(utils, 'find_resource')
    def test_instance_resize_volume(self, mock_find):
        args = ['instance1', '5']
        mock_find.side_effect = ['instance1']
        parsed_args = self.check_parser(self.cmd, args, [])
        result = self.cmd.take_action(parsed_args)
        self.instance_client.resize_volume.assert_called_with('instance1',
                                                              5)
        self.assertIsNone(result)


class TestDatabaseInstanceForceDelete(TestInstances):

    def setUp(self):
        super(TestDatabaseInstanceForceDelete, self).setUp()
        self.cmd = (database_instances
                    .ForceDeleteDatabaseInstance(self.app, None))

    @mock.patch.object(utils, 'find_resource')
    def test_instance_force_delete(self, mock_find):
        args = ['instance1']
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])
        result = self.cmd.take_action(parsed_args)
        self.instance_client.reset_status.assert_called_with('instance1')
        self.instance_client.delete.assert_called_with('instance1')
        self.assertIsNone(result)

    @mock.patch.object(utils, 'find_resource')
    def test_instance_force_delete_with_exception(self, mock_find):
        args = ['fakeinstance']
        parsed_args = self.check_parser(self.cmd, args, [])
        mock_find.return_value = args[0]

        self.instance_client.delete.side_effect = exceptions.CommandError
        self.assertRaises(exceptions.CommandError,
                          self.cmd.take_action,
                          parsed_args)


class TestDatabaseInstancePromoteToReplicaSource(TestInstances):

    def setUp(self):
        super(TestDatabaseInstancePromoteToReplicaSource, self).setUp()
        self.cmd = database_instances.PromoteDatabaseInstanceToReplicaSource(
            self.app, None)

    @mock.patch.object(utils, 'find_resource')
    def test_instance_promote_to_replica_source(self, mock_find):
        args = ['instance']
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])
        result = self.cmd.take_action(parsed_args)
        self.instance_client.promote_to_replica_source.assert_called_with(
            'instance')
        self.assertIsNone(result)


class TestDatabaseInstanceRestart(TestInstances):

    def setUp(self):
        super(TestDatabaseInstanceRestart, self).setUp()
        self.cmd = database_instances.RestartDatabaseInstance(self.app,
                                                              None)

    @mock.patch.object(utils, 'find_resource')
    def test_instance_restart(self, mock_find):
        args = ['instance1']
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])
        result = self.cmd.take_action(parsed_args)
        self.instance_client.restart.assert_called_with('instance1')
        self.assertIsNone(result)


class TestDatabaseInstanceEjectReplicaSource(TestInstances):

    def setUp(self):
        super(TestDatabaseInstanceEjectReplicaSource, self).setUp()
        self.cmd = database_instances.EjectDatabaseInstanceReplicaSource(
            self.app, None)

    @mock.patch.object(utils, 'find_resource')
    def test_eject_replica_source(self, mock_find):
        args = ['instance']
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])
        result = self.cmd.take_action(parsed_args)
        self.instance_client.eject_replica_source.assert_called_with(
            'instance')
        self.assertIsNone(result)


class TestDatabaseInstanceUpdate(TestInstances):

    def setUp(self):
        super(TestDatabaseInstanceUpdate, self).setUp()
        self.cmd = database_instances.UpdateDatabaseInstance(self.app, None)

    @mock.patch.object(utils, 'find_resource')
    def test_instance_update(self, mock_find):
        args = ['instance1',
                '--name', 'new_instance_name',
                '--detach_replica_source',
                '--remove_configuration']
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])
        result = self.cmd.take_action(parsed_args)
        self.instance_client.update.assert_called_with(
            'instance1',
            None,
            'new_instance_name',
            True, True,
            is_public=None, allowed_cidrs=None)
        self.assertIsNone(result)

    def test_instance_update_access(self):
        ins_id = '4c397f77-750d-43df-8fc5-f7388e4316ee'
        args = [ins_id,
                '--name', 'new_instance_name',
                '--is-private', '--allowed-cidr', '10.0.0.0/24',
                '--allowed-cidr', '10.0.1.0/24']
        parsed_args = self.check_parser(self.cmd, args, [])
        self.cmd.take_action(parsed_args)

        self.instance_client.update.assert_called_with(
            ins_id,
            None,
            'new_instance_name',
            False, False,
            is_public=False, allowed_cidrs=['10.0.0.0/24', '10.0.1.0/24'])


class TestInstanceReplicaDetach(TestInstances):

    def setUp(self):
        super(TestInstanceReplicaDetach, self).setUp()
        self.cmd = database_instances.DetachDatabaseInstanceReplica(
            self.app, None)

    @mock.patch.object(utils, 'find_resource')
    def test_instance_replica_detach(self, mock_find):
        args = ['instance']
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])
        result = self.cmd.take_action(parsed_args)
        self.instance_client.update.assert_called_with(
            'instance', detach_replica_source=True)
        self.assertIsNone(result)


class TestDatabaseInstanceReboot(TestInstances):
    def setUp(self):
        super(TestDatabaseInstanceReboot, self).setUp()
        self.cmd = database_instances.RebootDatabaseInstance(self.app, None)

    @mock.patch.object(utils, 'find_resource')
    def test_instance_restart(self, mock_find):
        args = ['instance1']
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])

        self.cmd.take_action(parsed_args)

        self.mgmt_client.reboot.assert_called_with('instance1')


class TestDatabaseInstanceRebuild(TestInstances):
    def setUp(self):
        super(TestDatabaseInstanceRebuild, self).setUp()
        self.cmd = database_instances.RebuildDatabaseInstance(self.app, None)

    @mock.patch.object(utils, 'find_resource')
    def test_instance_rebuild(self, mock_find):
        args = ['instance1', 'image_id']
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])

        self.cmd.take_action(parsed_args)

        self.mgmt_client.rebuild.assert_called_with('instance1', 'image_id')



# Copyright 2011 OpenStack Foundation
# Copyright 2013 Rackspace Hosting
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from troveclient import base


class Diagnostics(base.Resource):
    """Account is an opaque instance used to hold account information."""
    def __repr__(self):
        return "<Diagnostics: %s>" % self.version


class DiagnosticsInterrogator(base.ManagerWithFind):
    """Manager class for Interrogator resource."""
    resource_class = Diagnostics

    def get(self, instance):
        """Get the diagnostics of the guest on the instance."""
        return self._get("/mgmt/instances/%s/diagnostics" %
                         base.getid(instance), "diagnostics")

    # Appease the abc gods
    def list(self):
        pass


class HwInfo(base.Resource):

    def __repr__(self):
        return "<HwInfo: %s>" % self.version


class HwInfoInterrogator(base.ManagerWithFind):
    """Manager class for HwInfo."""
    resource_class = HwInfo

    def get(self, instance):
        """Get the hardware information of the instance."""
        return self._get("/mgmt/instances/%s/hwinfo" % base.getid(instance))

    # Appease the abc gods
    def list(self):
        pass



# Copyright 2011 OpenStack Foundation
# Copyright 2013 Rackspace Hosting
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import base64
import os
import simplejson as json
import sys
import uuid

from oslo_utils import encodeutils
from oslo_utils import uuidutils
import prettytable

from troveclient.apiclient import exceptions


def arg(*args, **kwargs):
    """Decorator for CLI args."""
    def _decorator(func):
        add_arg(func, *args, **kwargs)
        return func
    return _decorator


def env(*vars, **kwargs):
    """Returns environment variables.

    Returns the first environment variable set
    if none are non-empty, defaults to '' or keyword arg default
    """
    for v in vars:
        value = os.environ.get(v, None)
        if value:
            return value
    return kwargs.get('default', '')


def add_arg(f, *args, **kwargs):
    """Bind CLI arguments to a shell.py `do_foo` function."""

    if not hasattr(f, 'arguments'):
        f.arguments = []

    # NOTE(sirp): avoid dups that can occur when the module is shared across
    # tests.
    if (args, kwargs) not in f.arguments:
        # Because of the semantics of decorator composition if we just append
        # to the options list positional options will appear to be backwards.
        f.arguments.insert(0, (args, kwargs))


def unauthenticated(f):
    """Adds 'unauthenticated' attribute to decorated function.

    Usage::

        @unauthenticated
        def mymethod(f):
            ...

    """
    f.unauthenticated = True
    return f


def isunauthenticated(f):
    """Decorator to mark authentication-non-required.

    Checks to see if the function is marked as not requiring authentication
    with the @unauthenticated decorator. Returns True if decorator is
    set to True, False otherwise.
    """
    return getattr(f, 'unauthenticated', False)


def service_type(stype):
    """Adds 'service_type' attribute to decorated function.

    Usage::

        @service_type('database')
        def mymethod(f):
            ...

    """
    def inner(f):
        f.service_type = stype
        return f
    return inner


def get_service_type(f):
    """Retrieves service type from function."""
    return getattr(f, 'service_type', None)


def translate_keys(collection, convert):
    for item in collection:
        keys = list(item.__dict__.keys())
        for from_key, to_key in convert:
            if from_key in keys and to_key not in keys:
                setattr(item, to_key, item._info[from_key])


def _output_override(objs, print_as):
    """Output override flag checking.

    If an output override global flag is set, print with override
    raise BaseException if no printing was overridden.
    """
    if globals().get('json_output', False):
        if print_as == 'list':
            new_objs = []
            for o in objs:
                new_objs.append(o._info)
        elif print_as == 'dict':
            new_objs = objs
        # pretty print the json
        print(json.dumps(new_objs, indent='  '))
    else:
        raise BaseException('No valid output override')


def _print(pt, order):

    if sys.version_info >= (3, 0):
        print(pt.get_string(sortby=order))
    else:
        print(encodeutils.safe_encode(pt.get_string(sortby=order)))


def print_list(objs, fields, formatters={}, order_by=None, obj_is_dict=False,
               labels={}):
    try:
        _output_override(objs, 'list')
        return
    except BaseException:
        pass
    # Make nice labels from the fields, if not provided in the labels arg
    if not labels:
        labels = {}
    for field in fields:
        if field not in labels:
            # No underscores (use spaces instead) and uppercase any ID's
            label = field.replace("_", " ").replace(" id", " ID")
            # Uppercase anything else that's less than 3 chars
            if len(label) < 3:
                label = label.upper()
            # Capitalize each word otherwise
            else:
                label = ' '.join(word[0].upper() + word[1:]
                                 for word in label.split())
            labels[field] = label

    pt = prettytable.PrettyTable(
        [labels[field] for field in fields], caching=False)
    # set the default alignment to left-aligned
    align = dict((labels[field], 'l') for field in fields)
    set_align = True
    for obj in objs:
        row = []
        for field in fields:
            if formatters and field in formatters:
                data = formatters[field](obj)
            elif obj_is_dict:
                data = obj.get(field, '')
            else:
                data = getattr(obj, field, '')
            if isinstance(data, str):
                row.append(data.encode('utf-8'))
            else:
                row.append(str(data))
            # set the alignment to right-aligned if it's a numeric
            if set_align and hasattr(data, '__int__'):
                align[labels[field]] = 'r'
        set_align = False
        pt.add_row(row)
    pt._align = align

    if not order_by:
        order_by = fields[0]
    order_by = labels[order_by]
    _print(pt, order_by)


def print_dict(d, property="Property"):
    try:
        _output_override(d, 'dict')
        return
    except BaseException:
        pass
    pt = prettytable.PrettyTable([property, 'Value'], caching=False)
    pt.align = 'l'
    [pt.add_row(list(r)) for r in d.items()]
    _print(pt, property)


def get_resource_id(manager, id_or_name):
    if not uuidutils.is_uuid_like(id_or_name):
        try:
            id_or_name = get_resource_id_by_name(manager, id_or_name)
        except Exception as e:
            msg = ("Failed to get resource ID for %s, error: %s" %
                   (id_or_name, str(e)))
            raise exceptions.CommandError(msg)

    return id_or_name


def get_resource_id_by_name(manager, name):
    resource = manager.find(name=name)
    return resource.id


def find_resource(manager, name_or_id):
    """Helper for the _find_* methods."""
    # first try to get entity as integer id

    # When the 'name_or_id' is int, covert it to string.
    # Reason is that manager cannot find instance when name_or_id
    # is integer and instance name is digital.
    # Related to bug/1740015.
    if isinstance(name_or_id, int):
        name_or_id = str(name_or_id)
    elif sys.version_info <= (3, 0):
        name_or_id = encodeutils.safe_decode(name_or_id)

    try:
        return manager.get(name_or_id)
    except exceptions.NotFound:
        pass

    try:
        try:
            return manager.find(human_id=name_or_id)
        except exceptions.NotFound:
            pass

        # finally try to find entity by name
        try:
            return manager.find(name=name_or_id)
        except exceptions.NotFound:
            try:
                return manager.find(display_name=name_or_id)
            except (UnicodeDecodeError, exceptions.NotFound):
                try:
                    # Instances does not have name, but display_name
                    return manager.find(display_name=name_or_id)
                except exceptions.NotFound:
                    msg = "No %s with a name or ID of '%s' exists." % \
                        (manager.resource_class.__name__.lower(), name_or_id)
                    raise exceptions.CommandError(msg)
    except exceptions.NoUniqueMatch:
        msg = ("Multiple %s matches found for '%s', use an ID to be more"
               " specific." % (manager.resource_class.__name__.lower(),
                               name_or_id))
        raise exceptions.CommandError(msg)


def is_admin(cs):
    is_admin = False
    try:
        is_admin = 'admin' in cs.client.auth.auth_ref.role_names
    except Exception:
        print("Warning: Could not determine current role. Assuming non-admin")
        pass

    return is_admin


class HookableMixin(object):
    """Mixin so classes can register and run hooks."""
    _hooks_map = {}

    @classmethod
    def add_hook(cls, hook_type, hook_func):
        if hook_type not in cls._hooks_map:
            cls._hooks_map[hook_type] = []

        cls._hooks_map[hook_type].append(hook_func)

    @classmethod
    def run_hooks(cls, hook_type, *args, **kwargs):
        hook_funcs = cls._hooks_map.get(hook_type) or []
        for hook_func in hook_funcs:
            hook_func(*args, **kwargs)


def safe_issubclass(*args):
    """Like issubclass, but will just return False if not a class."""

    try:
        if issubclass(*args):
            return True
    except TypeError:
        pass

    return False


def is_uuid_like(val):
    """Returns validation of a value as a UUID.

    For our purposes, a UUID is a canonical form string:
    aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa

    """
    try:
        return str(uuid.UUID(val)) == val
    except (TypeError, ValueError, AttributeError):
        return False


def encode_data(data):
    """Encode the data using the base64 codec."""

    try:
        # py27str - if we've got text data, this should encode it
        # py27aa/py34aa - if we've got a bytearray, this should work too
        encoded = str(base64.b64encode(data).decode('utf-8'))
    except TypeError:
        # py34str - convert to bytes first, then we can encode
        data_bytes = bytes([ord(item) for item in data])
        encoded = base64.b64encode(data_bytes).decode('utf-8')

    return encoded


def decode_data(data):
    """Encode the data using the base64 codec."""

    # py27 & py34 seem to understand bytearray the same
    return bytearray([item for item in base64.b64decode(data)])


def do_action_with_msg(action, success_msg):
    """Helper to run an action with return message."""

    action
    print(success_msg)


def do_action_on_many(action, resources, success_msg, error_msg):
    """Helper to run an action on many resources."""
    failure_flag = False

    for resource in resources:
        try:
            action(resource)
            print(success_msg % resource)
        except Exception as e:
            failure_flag = True
            print(encodeutils.safe_encode(str(e)))

    if failure_flag:
        raise exceptions.CommandError(error_msg)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Database v1 Clusters action implementations"""

from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils

from troveclient.i18n import _
from troveclient.v1.shell import _parse_extended_properties
from troveclient.v1.shell import _parse_instance_options
from troveclient.v1.shell import EXT_PROPS_HELP
from troveclient.v1.shell import EXT_PROPS_METAVAR
from troveclient.v1.shell import INSTANCE_HELP
from troveclient.v1.shell import INSTANCE_METAVAR


def set_attributes_for_print_detail(cluster):
    info = cluster._info.copy()
    if hasattr(cluster, 'datastore'):
        info['datastore'] = cluster.datastore['type']
        info['datastore_version'] = cluster.datastore['version']
    if hasattr(cluster, 'task'):
        info['task_description'] = cluster.task['description']
        info['task_name'] = cluster.task['name']
    info.pop('task', None)
    if hasattr(cluster, 'ip'):
        info['ip'] = ', '.join(cluster.ip)
    instances = info.pop('instances', None)
    if instances:
        info['instance_count'] = len(instances)
    info.pop('links', None)
    return info


class ListDatabaseClusters(command.Lister):

    _description = _("List database clusters")
    columns = ['ID', 'Name', 'Datastore', 'Datastore Version',
               'Task Name']

    def get_parser(self, prog_name):
        parser = super(ListDatabaseClusters, self).get_parser(prog_name)
        parser.add_argument(
            '--limit',
            dest='limit',
            metavar='<limit>',
            type=int,
            default=None,
            help=_('Limit the number of results displayed.')
        )
        parser.add_argument(
            '--marker',
            dest='marker',
            metavar='<ID>',
            type=str,
            default=None,
            help=_('Begin displaying the results for IDs greater than the'
                   ' specified marker. When used with ``--limit``, set '
                   ' this to the last ID displayed in the previous run.')
        )
        return parser

    def take_action(self, parsed_args):
        database_clusters = self.app.client_manager.database.clusters
        clusters = database_clusters.list(limit=parsed_args.limit,
                                          marker=parsed_args.marker)
        for cluster in clusters:
            setattr(cluster, 'datastore_version',
                    cluster.datastore['version'])
            setattr(cluster, 'datastore', cluster.datastore['type'])
            setattr(cluster, 'task_name', cluster.task['name'])

        clusters = [utils.get_item_properties(c, self.columns)
                    for c in clusters]
        return self.columns, clusters


class ShowDatabaseCluster(command.ShowOne):
    _description = _("Shows details of a database cluster")

    def get_parser(self, prog_name):
        parser = super(ShowDatabaseCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or name of the cluster'),
        )
        return parser

    def take_action(self, parsed_args):
        database_clusters = self.app.client_manager.database.clusters
        cluster = utils.find_resource(database_clusters, parsed_args.cluster)
        cluster = set_attributes_for_print_detail(cluster)
        return zip(*sorted(cluster.items()))


class DeleteDatabaseCluster(command.Command):

    _description = _("Deletes a cluster.")

    def get_parser(self, prog_name):
        parser = super(DeleteDatabaseCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or name of the cluster.'),
        )
        return parser

    def take_action(self, parsed_args):
        database_clusters = self.app.client_manager.database.clusters
        try:
            cluster = utils.find_resource(database_clusters,
                                          parsed_args.cluster)
            database_clusters.delete(cluster)
        except Exception as e:
            msg = (_("Failed to delete cluster %(cluster)s: %(e)s")
                   % {'cluster': parsed_args.cluster, 'e': e})
            raise exceptions.CommandError(msg)


class CreateDatabaseCluster(command.ShowOne):
    _description = _("Creates a new database cluster.")

    def get_parser(self, prog_name):
        parser = super(CreateDatabaseCluster, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            type=str,
            help=_('Name of the cluster.'),
        )
        parser.add_argument(
            'datastore',
            metavar='<datastore>',
            help=_('A datastore name or ID.'),
        )
        parser.add_argument(
            'datastore_version',
            metavar='<datastore_version>',
            help=_('A datastore version name or ID.'),
        )
        parser.add_argument(
            '--instance',
            metavar=INSTANCE_METAVAR,
            action='append',
            dest='instances',
            default=[],
            help=INSTANCE_HELP,
        )
        parser.add_argument(
            '--locality',
            metavar='<policy>',
            default=None,
            choices=['affinity', 'anti-affinity'],
            help=_('Locality policy to use when creating cluster. '
                   'Choose one of %(choices)s.'),
        )
        parser.add_argument(
            '--extended-properties',
            dest='extended_properties',
            metavar=EXT_PROPS_METAVAR,
            default=None,
            help=EXT_PROPS_HELP,
        )
        parser.add_argument(
            '--configuration',
            metavar='<configuration>',
            type=str,
            default=None,
            help=_('ID of the configuration group to attach to the cluster.'),
        )
        return parser

    def take_action(self, parsed_args):
        database = self.app.client_manager.database
        instances = _parse_instance_options(database, parsed_args.instances)
        extended_properties = {}
        if parsed_args.extended_properties:
            extended_properties = _parse_extended_properties(
                parsed_args.extended_properties)
        cluster = database.clusters.create(
            parsed_args.name,
            parsed_args.datastore,
            parsed_args.datastore_version,
            instances=instances,
            locality=parsed_args.locality,
            extended_properties=extended_properties,
            configuration=parsed_args.configuration)
        cluster = set_attributes_for_print_detail(cluster)
        return zip(*sorted(cluster.items()))


class ResetDatabaseClusterStatus(command.Command):

    _description = _("Set the cluster task to NONE.")

    def get_parser(self, prog_name):
        parser = super(ResetDatabaseClusterStatus, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or name of the cluster.'),
        )
        return parser

    def take_action(self, parsed_args):
        database_clusters = self.app.client_manager.database.clusters
        cluster = utils.find_resource(database_clusters,
                                      parsed_args.cluster)
        database_clusters.reset_status(cluster)


class ListDatabaseClusterInstances(command.Lister):

    _description = _("Lists all instances of a cluster.")
    columns = ['ID', 'Name', 'Flavor ID', 'Size', 'Status']

    def get_parser(self, prog_name):
        parser = (super(ListDatabaseClusterInstances, self)
                  .get_parser(prog_name))
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or name of the cluster.'))
        return parser

    def take_action(self, parsed_args):
        database_clusters = self.app.client_manager.database.clusters
        cluster = utils.find_resource(database_clusters, parsed_args.cluster)
        instances = cluster._info['instances']
        for instance in instances:
            instance['flavor_id'] = instance['flavor']['id']
            if instance.get('volume'):
                instance['size'] = instance['volume']['size']

        instances = [utils.get_dict_properties(inst, self.columns)
                     for inst in instances]
        return self.columns, instances


class UpgradeDatabaseCluster(command.Command):

    _description = _("Upgrades a cluster to a new datastore version.")

    def get_parser(self, prog_name):
        parser = super(UpgradeDatabaseCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or name of the cluster.'),
        )
        parser.add_argument(
            'datastore_version',
            metavar='<datastore_version>',
            help=_('A datastore version name or ID.'),
        )
        return parser

    def take_action(self, parsed_args):
        database_clusters = self.app.client_manager.database.clusters
        cluster = utils.find_resource(database_clusters,
                                      parsed_args.cluster)
        database_clusters.upgrade(cluster, parsed_args.datastore_version)


class ForceDeleteDatabaseCluster(command.Command):

    _description = _("Force delete a cluster.")

    def get_parser(self, prog_name):
        parser = super(ForceDeleteDatabaseCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or name of the cluster.'),
        )
        return parser

    def take_action(self, parsed_args):
        database_clusters = self.app.client_manager.database.clusters
        cluster = utils.find_resource(database_clusters,
                                      parsed_args.cluster)
        database_clusters.reset_status(cluster)
        try:
            database_clusters.delete(cluster)
        except Exception as e:
            msg = (_("Failed to delete cluster %(cluster)s: %(e)s")
                   % {'cluster': parsed_args.cluster, 'e': e})
            raise exceptions.CommandError(msg)


class GrowDatabaseCluster(command.Command):

    _description = _("Adds more instances to a cluster.")

    def get_parser(self, prog_name):
        parser = super(GrowDatabaseCluster, self).get_parser(prog_name)
        parser.add_argument(
            '--instance',
            metavar=INSTANCE_METAVAR,
            action='append',
            dest='instances',
            default=[],
            help=INSTANCE_HELP
        )
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or name of the cluster.')
        )
        return parser

    def take_action(self, parsed_args):
        database_client_manager = self.app.client_manager.database

        db_clusters = database_client_manager.clusters
        cluster = utils.find_resource(db_clusters,
                                      parsed_args.cluster)

        instances = _parse_instance_options(database_client_manager,
                                            parsed_args.instances,
                                            for_grow=True)
        db_clusters.grow(cluster, instances=instances)


class ShrinkDatabaseCluster(command.Command):

    _description = _("Drops instances from a cluster.")

    def get_parser(self, prog_name):
        parser = super(ShrinkDatabaseCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or name of the cluster.')
        )
        parser.add_argument(
            'instances',
            metavar='<instance>',
            nargs='+',
            default=[],
            help=_("Drop instance(s) from the cluster. Specify "
                   "multiple ids to drop multiple instances.")
        )
        return parser

    def take_action(self, parsed_args):
        database_client_manager = self.app.client_manager.database

        db_clusters = database_client_manager.clusters
        cluster = utils.find_resource(db_clusters,
                                      parsed_args.cluster)

        db_instances = database_client_manager.instances
        instances = [
            {'id': utils.find_resource(db_instances,
                                       instance).id}
            for instance in parsed_args.instances
        ]
        db_clusters.shrink(cluster, instances)


class ListDatabaseClusterModules(command.Lister):

    _description = _("Lists all modules for each instance of a cluster.")
    columns = ['instance_name', 'Module Name', 'Module Type', 'md5',
               'created', 'updated']

    def get_parser(self, prog_name):
        parser = (super(ListDatabaseClusterModules, self)
                  .get_parser(prog_name))
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or name of the cluster.'))
        return parser

    def take_action(self, parsed_args):
        database_clusters = self.app.client_manager.database.clusters
        database_instances = self.app.client_manager.database.instances
        cluster = utils.find_resource(database_clusters, parsed_args.cluster)
        instances = cluster._info['instances']
        modules = []
        for instance in instances:
            new_list = database_instances.modules(instance['id'])
            for item in new_list:
                item.instance_id = instance['id']
                item.instance_name = instance['name']
                item.module_name = item.name
                item.module_type = item.type
            modules += new_list
        modules = [utils.get_item_properties(module, self.columns)
                   for module in modules]
        return self.columns, modules



# Copyright 2011 OpenStack Foundation
# Copyright 2013 Rackspace Hosting
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from troveclient import client as trove_client
from troveclient.v1 import backup_strategy
from troveclient.v1 import backups
from troveclient.v1 import clusters
from troveclient.v1 import configurations
from troveclient.v1 import databases
from troveclient.v1 import datastores
from troveclient.v1 import flavors
from troveclient.v1 import instances
from troveclient.v1 import limits
from troveclient.v1 import management
from troveclient.v1 import metadata
from troveclient.v1 import modules
from troveclient.v1 import quota
from troveclient.v1 import root
from troveclient.v1 import security_groups
from troveclient.v1 import users
from troveclient.v1 import volume_types


class Client(object):
    """Top-level object to access the OpenStack Database API.

    Create an instance with your creds::

        >> client = Client(USERNAME,
                           PASSWORD,
                           project_id=TENANT_NAME,
                           auth_url=AUTH_URL)

    Then call methods on its managers::

        >> client.instances.list()
        ...

    """

    def __init__(self, username=None, password=None, project_id=None,
                 auth_url='',
                 insecure=False, timeout=None, tenant_id=None,
                 proxy_tenant_id=None, proxy_token=None, region_name=None,
                 endpoint_type='publicURL', extensions=None,
                 service_type='database', service_name=None,
                 database_service_name=None, retries=None,
                 http_log_debug=False,
                 cacert=None, bypass_url=None,
                 auth_system='keystone', auth_plugin=None, session=None,
                 auth=None, **kwargs):
        # self.limits = limits.LimitsManager(self)

        # extensions
        self.flavors = flavors.Flavors(self)
        self.volume_types = volume_types.VolumeTypes(self)
        self.users = users.Users(self)
        self.databases = databases.Databases(self)
        self.backups = backups.Backups(self)
        self.backup_strategies = backup_strategy.BackupStrategiesManager(self)
        self.clusters = clusters.Clusters(self)
        self.instances = instances.Instances(self)
        self.limits = limits.Limits(self)
        self.root = root.Root(self)
        self.security_group_rules = security_groups.SecurityGroupRules(self)
        self.security_groups = security_groups.SecurityGroups(self)
        self.datastores = datastores.Datastores(self)
        self.datastore_versions = datastores.DatastoreVersions(self)
        self.configurations = configurations.Configurations(self)
        self.configuration_parameters = configurations.ConfigurationParameters(
            self)
        self.metadata = metadata.Metadata(self)
        self.modules = modules.Modules(self)
        self.quota = quota.Quotas(self)
        self.mgmt_instances = management.Management(self)
        self.mgmt_ds_versions = management.MgmtDatastoreVersions(self)

        # Add in any extensions...
        if extensions:
            for extension in extensions:
                if extension.manager_class:
                    setattr(self, extension.name,
                            extension.manager_class(self))

        self.client = trove_client._construct_http_client(
            username=username,
            password=password,
            project_id=project_id,
            auth_url=auth_url,
            insecure=insecure,
            timeout=timeout,
            tenant_id=tenant_id,
            proxy_token=proxy_token,
            proxy_tenant_id=proxy_tenant_id,
            region_name=region_name,
            endpoint_type=endpoint_type,
            service_type=service_type,
            service_name=service_name,
            database_service_name=database_service_name,
            retries=retries,
            http_log_debug=http_log_debug,
            cacert=cacert,
            bypass_url=bypass_url,
            auth_system=auth_system,
            auth_plugin=auth_plugin,
            session=session,
            auth=auth,
            **kwargs)

    def authenticate(self):
        """Authenticate against the server.

        Normally this is called automatically when you first access the API,
        but you can call this method to force authentication right now.

        Returns on success; raises :exc:`exceptions.Unauthorized` if the
        credentials are wrong.
        """
        self.client.authenticate()

    def get_database_api_version_from_endpoint(self):
        return self.client.get_database_api_version_from_endpoint()



# Copyright 2010 Jacob Kaplan-Moss
# Copyright 2012 OpenStack Foundation
# Copyright 2013 Rackspace Hosting
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Base utilities to build API operation managers and objects on top of.
"""
import abc
import contextlib
import hashlib
import os

from urllib import parse

from troveclient.apiclient import base
from troveclient.apiclient import exceptions
from troveclient import common
from troveclient import utils

# Python 2.4 compat
try:
    all
except NameError:
    def all(iterable):
        return True not in (not x for x in iterable)


def getid(obj):
    """Retrieves an id from object or integer.

    Abstracts the common pattern of allowing both an object or an object's
    ID as a parameter when dealing with relationships.
    """
    try:
        return obj.id
    except AttributeError:
        return obj


class Manager(utils.HookableMixin):
    """Manager defining CRUD operations for API.

    Managers interact with a particular type of API (servers, flavors,
    images, etc.) and provide CRUD operations for them.
    """
    resource_class = None

    def __init__(self, api):
        self.api = api

    def _paginated(self, url, response_key, limit=None, marker=None,
                   query_strings=None):
        query_strings = query_strings or {}
        url = common.append_query_strings(url, limit=limit, marker=marker,
                                          **query_strings)
        resp, body = self.api.client.get(url)
        if not body:
            raise Exception("Call to " + url + " did not return a body.")
        links = body.get('links', [])
        next_links = [link['href'] for link in links if link['rel'] == 'next']
        next_marker = None
        for link in next_links:
            # Extract the marker from the url.
            parsed_url = parse.urlparse(link)
            query_dict = dict(parse.parse_qsl(parsed_url.query))
            next_marker = query_dict.get('marker')
        data = [self.resource_class(self, res) for res in body[response_key]]
        return common.Paginated(data, next_marker=next_marker, links=links)

    def _list(self, url, response_key, obj_class=None, body=None):
        resp = None
        if body:
            resp, body = self.api.client.post(url, body=body)
        else:
            resp, body = self.api.client.get(url)

        if obj_class is None:
            obj_class = self.resource_class

        data = body[response_key]
        # NOTE(ja): keystone returns values as list as {'values': [ ... ]}
        #           unlike other services which just return the list...
        if isinstance(data, dict):
            try:
                data = data['values']
            except KeyError:
                pass

        with self.completion_cache('human_id', obj_class, mode="w"):
            with self.completion_cache('uuid', obj_class, mode="w"):
                return [obj_class(self, res, loaded=True)
                        for res in data if res]

    @contextlib.contextmanager
    def completion_cache(self, cache_type, obj_class, mode):
        """Bash-completion cache.

        The completion cache store items that can be used for bash
        autocompletion, like UUIDs or human-friendly IDs.

        A resource listing will clear and repopulate the cache.

        A resource create will append to the cache.

        Delete is not handled because listings are assumed to be performed
        often enough to keep the cache reasonably up-to-date.
        """
        base_dir = utils.env('TROVECLIENT_UUID_CACHE_DIR',
                             default="~/.troveclient")

        # NOTE(sirp): Keep separate UUID caches for each username + endpoint
        # pair
        username = utils.env('OS_USERNAME', 'TROVE_USERNAME')
        url = utils.env('OS_URL', 'NOVA_URL')
        uniqifier = hashlib.md5(username.encode('utf-8') +
                                url.encode('utf-8')).hexdigest()

        cache_dir = os.path.expanduser(os.path.join(base_dir, uniqifier))

        try:
            os.makedirs(cache_dir, 0o755)
        except OSError:
            # NOTE(kiall): This is typically either permission denied while
            #              attempting to create the directory, or the directory
            #              already exists. Either way, don't fail.
            pass

        resource = obj_class.__name__.lower()
        filename = "%s-%s-cache" % (resource, cache_type.replace('_', '-'))
        path = os.path.join(cache_dir, filename)

        cache_attr = "_%s_cache" % cache_type

        try:
            setattr(self, cache_attr, open(path, mode))
        except IOError:
            # NOTE(kiall): This is typically a permission denied while
            #              attempting to write the cache file.
            pass

        try:
            yield
        finally:
            cache = getattr(self, cache_attr, None)
            if cache:
                cache.close()
                delattr(self, cache_attr)

    def write_to_completion_cache(self, cache_type, val):
        cache = getattr(self, "_%s_cache" % cache_type, None)
        if cache:
            cache.write("%s\n" % val)

    def _get(self, url, response_key=None):
        resp, body = self.api.client.get(url)
        if response_key:
            return self.resource_class(self, body[response_key], loaded=True)
        else:
            return self.resource_class(self, body, loaded=True)

    def _create(self, url, body, response_key, return_raw=False, **kwargs):
        self.run_hooks('modify_body_for_create', body, **kwargs)
        resp, body = self.api.client.post(url, body=body)
        if body:
            if return_raw:
                return body[response_key]

            with self.completion_cache('human_id', self.resource_class,
                                       mode="a"):
                with self.completion_cache('uuid', self.resource_class,
                                           mode="a"):
                    return self.resource_class(self, body[response_key])

    def _delete(self, url):
        resp, body = self.api.client.delete(url)
        return resp, body

    def _update(self, url, body, **kwargs):
        self.run_hooks('modify_body_for_update', body, **kwargs)
        resp, body = self.api.client.put(url, body=body)
        return body

    def _edit(self, url, body):
        resp, body = self.api.client.patch(url, body=body)
        return body


class ManagerWithFind(Manager, metaclass=abc.ABCMeta):
    """Like a `Manager`, but with additional `find()`/`findall()` methods."""

    @abc.abstractmethod
    def list(self):
        pass

    def find(self, **kwargs):
        """Find a single item with attributes matching ``**kwargs``.

        This isn't very efficient: it loads the entire list then filters on
        the Python side.
        """
        matches = self.findall(**kwargs)
        num_matches = len(matches)
        if num_matches == 0:
            msg = "No %s matching %s." % (self.resource_class.__name__, kwargs)
            raise exceptions.NotFound(404, msg)
        elif num_matches > 1:
            raise exceptions.NoUniqueMatch
        else:
            return matches[0]

    def findall(self, **kwargs):
        """Find all items with attributes matching ``**kwargs``.

        This isn't very efficient: it loads the entire list then filters on
        the Python side.
        """
        found = []
        searches = list(kwargs.items())

        for obj in self.list():
            try:
                if all(getattr(obj, attr) == value
                       for (attr, value) in searches):
                    found.append(self.get(obj.id))
            except AttributeError:
                continue

        return found


class Resource(base.Resource):
    """A resource represents a particular instance of an object like server.

    This is pretty much just a bag for attributes.
    :param manager: Manager object
    :param info: dictionary representing resource attributes
    :param loaded: prevent lazy-loading if set to True
    """
    HUMAN_ID = False

    def __init__(self, manager, info, loaded=False):
        super(Resource, self).__init__(manager, info, loaded)

        # NOTE(sirp): ensure `id` is already present because if it isn't we'll
        # enter an infinite loop of __getattr__ -> get -> __init__ ->
        # __getattr__ -> ...
        if 'id' in self.__dict__ and len(str(self.id)) == 36:
            self.manager.write_to_completion_cache('uuid', self.id)

        human_id = self.human_id
        if human_id:
            self.manager.write_to_completion_cache('human_id', human_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from osc_lib import utils

from troveclient.osc.v1 import database_logs
from troveclient.tests.osc.v1 import fakes


class TestLogs(fakes.TestDatabasev1):
    fake_logs = fakes.FakeLogs()

    def setUp(self):
        super(TestLogs, self).setUp()
        self.instance_client = self.app.client_manager.database.instances


class TestLogList(TestLogs):

    columns = database_logs.ListDatabaseLogs.columns
    values = [('general', 'USER', 'Partial', '128', '4096', 'data_logs',
               'mysql-general'),
              ('slow_query', 'USER', 'Ready', '0', '128', 'None', 'None')]

    def setUp(self):
        super(TestLogList, self).setUp()
        self.cmd = database_logs.ListDatabaseLogs(self.app, None)
        data = self.fake_logs.get_logs()
        self.instance_client.log_list.return_value = data

    @mock.patch.object(utils, 'find_resource')
    def test_log_list(self, mock_find):
        args = ['instance']
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.values, data)


class TestShowDatabaseInstanceLog(TestLogs):

    def setUp(self):
        super(TestShowDatabaseInstanceLog, self).setUp()
        self.cmd = database_logs.ShowDatabaseInstanceLog(self.app, None)
        self.columns = (
            'container',
            'metafile',
            'name',
            'pending',
            'prefix',
            'published',
            'status',
            'type',
        )

    @mock.patch.object(utils, 'find_resource')
    def test_show_instance_log(self, mock_find):
        mock_find.return_value = 'fake_instance_id'
        data = self.fake_logs.get_logs()[0]
        self.instance_client.log_show.return_value = data

        args = ['instance', 'logname']
        parsed_args = self.check_parser(self.cmd, args, [])
        columns, values = self.cmd.take_action(parsed_args)

        self.assertEqual(self.columns, columns)
        self.assertCountEqual(data.to_dict().values(), values)


class TestSetDatabaseInstanceLog(TestLogs):
    def setUp(self):
        super(TestSetDatabaseInstanceLog, self).setUp()
        self.cmd = database_logs.SetDatabaseInstanceLog(self.app, None)

    @mock.patch.object(utils, 'find_resource')
    def test_set_instance_log(self, mock_find):
        mock_find.return_value = 'fake_instance_id'
        data = self.fake_logs.get_logs()[0]
        data.status = 'Ready'
        self.instance_client.log_action.return_value = data

        args = ['instance1', 'log_name', '--enable']
        parsed_args = self.check_parser(self.cmd, args, [])
        self.cmd.take_action(parsed_args)

        self.instance_client.log_action.assert_called_once_with(
            'fake_instance_id', 'log_name',
            enable=True,
            disable=False,
            discard=False,
            publish=False
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

from unittest import mock

from osc_lib import exceptions
from osc_lib import utils

from troveclient.osc.v1 import database_root
from troveclient.tests.osc.v1 import fakes


class TestRoot(fakes.TestDatabasev1):
    fake_root = fakes.FakeRoot()

    def setUp(self):
        super(TestRoot, self).setUp()
        self.mock_client = self.app.client_manager.database
        self.root_client = self.app.client_manager.database.root


class TestRootEnable(TestRoot):

    def setUp(self):
        super(TestRootEnable, self).setUp()
        self.cmd = database_root.EnableDatabaseRoot(self.app, None)
        self.data = {
            'instance': self.fake_root.post_instance_1234_root(),
            'cluster': self.fake_root.post_cls_1234_root()
        }
        self.columns = ('name', 'password',)

    @mock.patch.object(utils, 'find_resource')
    def test_enable_instance_1234_root(self, mock_find):
        self.root_client.create_instance_root.return_value = (
            self.data['instance'])
        args = ['1234']
        parsed_args = self.check_parser(self.cmd, args, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.assertEqual(self.columns, columns)
        self.assertEqual(('root', 'password',), data)

    @mock.patch.object(utils, 'find_resource')
    def test_enable_cluster_1234_root(self, mock_find):
        mock_find.side_effect = [exceptions.CommandError(),
                                 (None, 'cluster')]
        self.root_client.create_cluster_root.return_value = (
            self.data['cluster'])
        args = ['1234']
        parsed_args = self.check_parser(self.cmd, args, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.assertEqual(self.columns, columns)
        self.assertEqual(('root', 'password',), data)

    @mock.patch.object(utils, 'find_resource')
    def test_enable_instance_root_with_password(self, mock_find):
        args = ['1234', '--root_password', 'secret']
        parsed_args = self.check_parser(self.cmd, args, [])
        self.cmd.take_action(parsed_args)
        self.root_client.create_instance_root(None,
                                              root_password='secret')

    @mock.patch.object(utils, 'find_resource')
    def test_enable_cluster_root_with_password(self, mock_find):
        args = ['1234', '--root_password', 'secret']
        parsed_args = self.check_parser(self.cmd, args, [])
        self.cmd.take_action(parsed_args)
        self.root_client.create_cluster_root(None,
                                             root_password='secret')


class TestRootDisable(TestRoot):

    def setUp(self):
        super(TestRootDisable, self).setUp()
        self.cmd = database_root.DisableDatabaseRoot(self.app, None)
        self.data = self.fake_root.delete_instance_1234_root()

    @mock.patch.object(utils, 'find_resource')
    def test_disable_instance_1234_root(self, mock_find):
        self.root_client.disable_instance_root.return_value = self.data
        args = ['1234']
        mock_find.return_value = args[0]
        parsed_args = self.check_parser(self.cmd, args, [])
        result = self.cmd.take_action(parsed_args)
        self.root_client.disable_instance_root.assert_called_with('1234')
        self.assertIsNone(result)


class TestRootShow(TestRoot):

    def setUp(self):
        super(TestRootShow, self).setUp()
        self.cmd = database_root.ShowDatabaseRoot(self.app, None)
        self.data = {
            'instance': self.fake_root.get_instance_1234_root(),
            'cluster': self.fake_root.get_cls_1234_root()
        }
        self.columns = ('is_root_enabled',)

    @mock.patch.object(utils, 'find_resource')
    def test_show_instance_1234_root(self, mock_find):
        self.root_client.is_instance_root_enabled.return_value = (
            self.data['instance'])
        args = ['1234']
        parsed_args = self.check_parser(self.cmd, args, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.assertEqual(self.columns, columns)
        self.assertEqual(('True',), data)

    @mock.patch.object(utils, 'find_resource')
    def test_show_cluster_1234_root(self, mock_find):
        mock_find.side_effect = [exceptions.CommandError(),
                                 (None, 'cluster')]
        self.root_client.is_cluster_root_enabled.return_value = (
            self.data['cluster'])
        args = ['1234']
        parsed_args = self.check_parser(self.cmd, args, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.assertEqual(self.columns, columns)
        self.assertEqual(('True',), data)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
from oslo_utils import uuidutils

from troveclient import common
from troveclient import exceptions
from troveclient.osc.v1 import datastores
from troveclient.tests.osc.v1 import fakes


class TestDatastores(fakes.TestDatabasev1):
    fake_datastores = fakes.FakeDatastores()

    def setUp(self):
        super(TestDatastores, self).setUp()
        self.datastore_client = self.app.client_manager.database.datastores
        self.datastore_version_client =\
            self.app.client_manager.database.datastore_versions
        self.dsversion_mgmt_client =\
            self.app.client_manager.database.mgmt_ds_versions


class TestDatastoreList(TestDatastores):
    columns = datastores.ListDatastores.columns
    values = ('d-123', 'mysql')

    def setUp(self):
        super(TestDatastoreList, self).setUp()
        self.cmd = datastores.ListDatastores(self.app, None)
        data = [self.fake_datastores.get_datastores_d_123()]
        self.datastore_client.list.return_value = common.Paginated(data)

    def test_datastore_list_defaults(self):
        parsed_args = self.check_parser(self.cmd, [], [])
        columns, data = self.cmd.take_action(parsed_args)
        self.datastore_client.list.assert_called_once_with()
        self.assertEqual(self.columns, columns)
        self.assertEqual([self.values], data)


class TestDatastoreShow(TestDatastores):

    values = ('5.6', 'd-123', 'mysql', '5.6 (v-56)')

    def setUp(self):
        super(TestDatastoreShow, self).setUp()
        self.cmd = datastores.ShowDatastore(self.app, None)
        self.data = self.fake_datastores.get_datastores_d_123()
        self.datastore_client.get.return_value = self.data
        self.columns = (
            'default_version',
            'id',
            'name',
            'versions (id)',
        )

    def test_show(self):
        args = ['mysql']
        parsed_args = self.check_parser(self.cmd, args, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.values, data)


class TestDeleteDatastore(TestDatastores):
    def setUp(self):
        super(TestDeleteDatastore, self).setUp()
        self.cmd = datastores.DeleteDatastore(self.app, None)

    def test_delete_datastore(self):
        ds_id = uuidutils.generate_uuid()
        args = [ds_id]
        parsed_args = self.check_parser(self.cmd, args, [])

        self.cmd.take_action(parsed_args)

        self.datastore_client.delete.assert_called_once_with(ds_id)


class TestDatastoreVersionList(TestDatastores):
    columns = datastores.ListDatastoreVersions.columns
    values = ('v-56', '5.6', '')

    def setUp(self):
        super(TestDatastoreVersionList, self).setUp()
        self.cmd = datastores.ListDatastoreVersions(self.app, None)
        self.data = [self.fake_datastores.get_datastores_d_123_versions()]
        self.datastore_version_client.list.return_value =\
            common.Paginated(self.data)

    def test_datastore_version_list_defaults(self):
        args = ['mysql']
        parsed_args = self.check_parser(self.cmd, args, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.datastore_version_client.list.assert_called_once_with(args[0])
        self.assertEqual(self.columns, columns)
        self.assertEqual([self.values], data)


class TestDatastoreVersionShow(TestDatastores):
    values = ('v-56', '5.6')

    def setUp(self):
        super(TestDatastoreVersionShow, self).setUp()
        self.cmd = datastores.ShowDatastoreVersion(self.app, None)
        self.data = self.fake_datastores.get_datastores_d_123_versions()
        self.datastore_version_client.get.return_value = self.data
        self.columns = (
            'id',
            'name',
        )

    def test_datastore_version_show_defaults(self):
        args = ['5.6', '--datastore', 'mysql']
        verifylist = [
            ('datastore_version', '5.6'),
            ('datastore', 'mysql'),
        ]
        parsed_args = self.check_parser(self.cmd, args, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.values, data)

    def test_datastore_version_show_with_version_id_exception(self):
        args = [
            'v-56',
        ]
        verifylist = [
            ('datastore_version', 'v-56'),
        ]
        parsed_args = self.check_parser(self.cmd, args, verifylist)
        self.assertRaises(exceptions.NoUniqueMatch,
                          self.cmd.take_action,
                          parsed_args)


class TestDeleteDatastoreVersion(TestDatastores):
    def setUp(self):
        super(TestDeleteDatastoreVersion, self).setUp()
        self.cmd = datastores.DeleteDatastoreVersion(self.app, None)

    def test_delete_datastore_version(self):
        dsversion_id = uuidutils.generate_uuid()
        args = [dsversion_id]
        parsed_args = self.check_parser(self.cmd, args, [])

        self.cmd.take_action(parsed_args)

        self.dsversion_mgmt_client.delete.assert_called_once_with(dsversion_id)


class TestCreateDatastoreVersion(TestDatastores):
    def setUp(self):
        super(TestCreateDatastoreVersion, self).setUp()
        self.cmd = datastores.CreateDatastoreVersion(self.app, None)

    def test_create_datastore_version(self):
        image_id = uuidutils.generate_uuid()
        args = ['new_name', 'ds_name', 'ds_manager', image_id, '--active',
                '--default', '--image-tags', 'trove,mysql']
        parsed_args = self.check_parser(self.cmd, args, [])

        self.cmd.take_action(parsed_args)

        self.dsversion_mgmt_client.create.assert_called_once_with(
            'new_name', 'ds_name', 'ds_manager', image_id, active='true',
            default='true', image_tags=['trove', 'mysql'],
            version=None)


class TestUpdateDatastoreVersion(TestDatastores):
    def setUp(self):
        super(TestUpdateDatastoreVersion, self).setUp()
        self.cmd = datastores.UpdateDatastoreVersion(self.app, None)

    def test_update_datastore_version(self):
        version_id = uuidutils.generate_uuid()
        args = [version_id, '--image-tags', 'trove,mysql', '--enable',
                '--non-default']
        parsed_args = self.check_parser(self.cmd, args, [])

        self.cmd.take_action(parsed_args)

        self.dsversion_mgmt_client.edit.assert_called_once_with(
            version_id, datastore_manager=None, image=None,
            active='true', default='false', image_tags=['trove', 'mysql'],
            name=None)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Database v1 Backups action implementations"""

from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils as osc_utils
from oslo_utils import uuidutils

from troveclient.i18n import _
from troveclient.osc.v1 import base
from troveclient import utils as trove_utils


def set_attributes_for_print_detail(backup):
    info = backup._info.copy()
    if hasattr(backup, 'datastore'):
        info['datastore'] = backup.datastore['type']
        info['datastore_version'] = backup.datastore['version']
        info['datastore_version_id'] = backup.datastore['version_id']
    return info


class ListDatabaseBackups(command.Lister):

    _description = _("List database backups")
    columns = ['ID', 'Instance ID', 'Name', 'Status', 'Parent ID',
               'Updated', 'Project ID']

    def get_parser(self, prog_name):
        parser = super(ListDatabaseBackups, self).get_parser(prog_name)
        parser.add_argument(
            '--limit',
            dest='limit',
            metavar='<limit>',
            default=None,
            help=_('Return up to N number of the most recent bcakups.')
        )
        parser.add_argument(
            '--marker',
            dest='marker',
            metavar='<ID>',
            type=str,
            default=None,
            help=_('Begin displaying the results for IDs greater than the'
                   'specified marker. When used with ``--limit``, set '
                   'this to the last ID displayed in the previous run.')
        )
        parser.add_argument(
            '--datastore',
            dest='datastore',
            metavar='<datastore>',
            default=None,
            help=_('ID or name of the datastore (to filter backups by).')
        )
        parser.add_argument(
            '--instance-id',
            default=None,
            help=_('Filter backups by database instance ID.')
        )
        parser.add_argument(
            '--all-projects',
            action='store_true',
            help=_('Get all the backups of all the projects(Admin only).')
        )
        parser.add_argument(
            '--project-id',
            default=None,
            help=_('Filter backups by project ID.')
        )
        return parser

    def take_action(self, parsed_args):
        database_backups = self.app.client_manager.database.backups
        items = database_backups.list(limit=parsed_args.limit,
                                      datastore=parsed_args.datastore,
                                      marker=parsed_args.marker,
                                      instance_id=parsed_args.instance_id,
                                      all_projects=parsed_args.all_projects,
                                      project_id=parsed_args.project_id)
        backups = items
        while items.next and not parsed_args.limit:
            items = database_backups.list(
                marker=items.next,
                datastore=parsed_args.datastore,
                instance_id=parsed_args.instance_id,
                all_projects=parsed_args.all_projects,
                project_id=parsed_args.project_id
            )
            backups += items

        backups = [osc_utils.get_item_properties(b, self.columns)
                   for b in backups]

        return self.columns, backups


class ListDatabaseInstanceBackups(command.Lister):

    _description = _("Lists available backups for an instance.")
    columns = ['ID', 'Instance ID', 'Name', 'Status', 'Parent ID',
               'Updated']

    def get_parser(self, prog_name):
        parser = super(ListDatabaseInstanceBackups, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or name of the instance.')
        )
        parser.add_argument(
            '--limit',
            dest='limit',
            metavar='<limit>',
            default=None,
            help=_('Return up to N number of the most recent bcakups.')
        )
        parser.add_argument(
            '--marker',
            dest='marker',
            metavar='<ID>',
            type=str,
            default=None,
            help=_('Begin displaying the results for IDs greater than the'
                   'specified marker. When used with ``--limit``, set '
                   'this to the last ID displayed in the previous run.')
        )
        return parser

    def take_action(self, parsed_args):
        database_instances = self.app.client_manager.database.instances
        instance = osc_utils.find_resource(database_instances,
                                           parsed_args.instance)
        items = database_instances.backups(instance, limit=parsed_args.limit,
                                           marker=parsed_args.marker)
        backups = items
        while items.next and not parsed_args.limit:
            items = database_instances.backups(instance, marker=items.next)
            backups += items
        backups = [osc_utils.get_item_properties(b, self.columns)
                   for b in backups]
        return self.columns, backups


class ShowDatabaseBackup(command.ShowOne):

    _description = _("Shows details of a database backup")

    def get_parser(self, prog_name):
        parser = super(ShowDatabaseBackup, self).get_parser(prog_name)
        parser.add_argument(
            'backup',
            metavar='<backup>',
            help=_('ID or name of the backup'),
        )
        return parser

    def take_action(self, parsed_args):
        database_backups = self.app.client_manager.database.backups
        backup = osc_utils.find_resource(database_backups, parsed_args.backup)
        backup = set_attributes_for_print_detail(backup)
        return zip(*sorted(backup.items()))


class DeleteDatabaseBackup(base.TroveDeleter):

    _description = _("Deletes a backup.")

    def get_parser(self, prog_name):
        parser = super(DeleteDatabaseBackup, self).get_parser(prog_name)
        parser.add_argument(
            'backup',
            nargs='+',
            metavar='backup',
            help='Id or name of backup(s).'
        )
        return parser

    def take_action(self, parsed_args):
        db_backups = self.app.client_manager.database.backups

        # Used for batch deletion
        self.delete_func = db_backups.delete
        self.resource = 'database backup'

        ids = []
        for backup_id in parsed_args.backup:
            if not uuidutils.is_uuid_like(backup_id):
                try:
                    backup_id = trove_utils.get_resource_id_by_name(
                        db_backups, backup_id
                    )
                except Exception as e:
                    msg = ("Failed to get database backup %s, error: %s" %
                           (backup_id, str(e)))
                    raise exceptions.CommandError(msg)

            ids.append(backup_id)

        self.delete_resources(ids)


class CreateDatabaseBackup(command.ShowOne):

    _description = _("Creates a backup of an instance.")

    def get_parser(self, prog_name):
        parser = super(CreateDatabaseBackup, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of the backup.')
        )
        parser.add_argument(
            '-i',
            '--instance',
            metavar='<instance>',
            help=_('ID or name of the instance. This is not required if '
                   'restoring a backup from the data location.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            default=None,
            help=_('An optional description for the backup.')
        )
        parser.add_argument(
            '--parent',
            metavar='<parent>',
            default=None,
            help=_('Optional ID of the parent backup to perform an'
                   ' incremental backup from.')
        )
        parser.add_argument(
            '--incremental',
            action='store_true',
            default=False,
            help=_('Create an incremental backup based on the last'
                   ' full or incremental backup. It will create a'
                   ' full backup if no existing backup found.')
        )
        parser.add_argument(
            '--swift-container',
            help=_('The container name for storing the backup data when Swift '
                   'is used as backup storage backend. If not specified, will '
                   'use the container name configured in the backup strategy, '
                   'otherwise, the default value configured by the cloud '
                   'operator. Non-existent container is created '
                   'automatically.')
        )
        parser.add_argument(
            '--restore-from',
            help=_('The original backup data location, typically this is a '
                   'Swift object URL.')
        )
        parser.add_argument(
            '--restore-datastore-version',
            help=_('ID of the local datastore version corresponding to the '
                   'original backup')
        )
        parser.add_argument(
            '--restore-size', type=float,
            help=_('The original backup size.')
        )
        return parser

    def take_action(self, parsed_args):
        manager = self.app.client_manager.database
        database_backups = manager.backups
        params = {}
        instance_id = None

        if parsed_args.restore_from:
            # Leave the input validation to Trove server.
            params.update({
                'restore_from': parsed_args.restore_from,
                'restore_ds_version': parsed_args.restore_datastore_version,
                'restore_size': parsed_args.restore_size,
            })
        elif not parsed_args.instance:
            raise exceptions.CommandError('Instance ID or name is required if '
                                          'not restoring a backup.')
        else:
            instance_id = trove_utils.get_resource_id(manager.instances,
                                                      parsed_args.instance)
            params.update({
                'description': parsed_args.description,
                'parent_id': parsed_args.parent,
                'incremental': parsed_args.incremental,
                'swift_container': parsed_args.swift_container
            })

        backup = database_backups.create(parsed_args.name, instance_id,
                                         **params)
        backup = set_attributes_for_print_detail(backup)
        return zip(*sorted(backup.items()))


class DeleteDatabaseBackupExecution(command.Command):

    _description = _("Deletes an execution.")

    def get_parser(self, prog_name):
        parser = super(DeleteDatabaseBackupExecution, self).get_parser(
            prog_name)
        parser.add_argument(
            'execution',
            metavar='<execution>',
            help=_('ID of the execution to delete.')
        )
        return parser

    def take_action(self, parsed_args):
        database_backups = self.app.client_manager.database.backups
        database_backups.execution_delete(parsed_args.execution)



# Copyright 2011 OpenStack Foundation
# Copyright 2013 Rackspace Hosting
# Copyright 2013 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools
from unittest import mock

from troveclient import base
from troveclient.v1 import management

"""
Unit tests for management.py
"""


class RootHistoryTest(testtools.TestCase):

    def setUp(self):
        super(RootHistoryTest, self).setUp()
        self.orig__init = management.RootHistory.__init__
        management.RootHistory.__init__ = mock.Mock(return_value=None)

    def tearDown(self):
        super(RootHistoryTest, self).tearDown()
        management.RootHistory.__init__ = self.orig__init

    def test___repr__(self):
        root_history = management.RootHistory()
        root_history.id = "1"
        root_history.created = "ct"
        root_history.user = "tu"
        self.assertEqual('<Root History: Instance 1 enabled at ct by tu>',
                         root_history.__repr__())


class ManagementTest(testtools.TestCase):

    def setUp(self):
        super(ManagementTest, self).setUp()
        self.orig__init = management.Management.__init__
        management.Management.__init__ = mock.Mock(return_value=None)
        self.management = management.Management()
        self.management.api = mock.Mock()
        self.management.api.client = mock.Mock()

        self.orig_hist__init = management.RootHistory.__init__
        self.orig_base_getid = base.getid
        base.getid = mock.Mock(return_value="instance1")

    def tearDown(self):
        super(ManagementTest, self).tearDown()
        management.Management.__init__ = self.orig__init
        management.RootHistory.__init__ = self.orig_hist__init
        base.getid = self.orig_base_getid

    def test_show(self):
        def side_effect_func(path, instance):
            return path, instance
        self.management._get = mock.Mock(side_effect=side_effect_func)
        p, i = self.management.show(1)
        self.assertEqual(('/mgmt/instances/instance1', 'instance'), (p, i))

    def test_list(self):
        page_mock = mock.Mock()
        self.management._paginated = page_mock

        self.management.list(deleted=True)
        page_mock.assert_called_with('/mgmt/instances', 'instances', None,
                                     None, query_strings={'deleted': True})

        self.management.list(deleted=False, limit=10, marker="foo")
        page_mock.assert_called_with('/mgmt/instances', 'instances', 10, "foo",
                                     query_strings={"deleted": False})

    def test_index(self):
        """index() is just wrapper for list()"""
        page_mock = mock.Mock()
        self.management._paginated = page_mock

        self.management.index(deleted=True)
        page_mock.assert_called_with('/mgmt/instances', 'instances', None,
                                     None, query_strings={'deleted': True})

        self.management.index(deleted=False, limit=10, marker="foo")
        page_mock.assert_called_with('/mgmt/instances', 'instances', 10, "foo",
                                     query_strings={"deleted": False})

    def test_root_enabled_history(self):
        self.management.api.client.get = mock.Mock(return_value=('resp', None))
        self.assertRaises(Exception,
                          self.management.root_enabled_history, "instance")
        body = {'root_history': 'rh'}
        self.management.api.client.get = mock.Mock(return_value=('resp', body))
        management.RootHistory.__init__ = mock.Mock(return_value=None)
        rh = self.management.root_enabled_history("instance")
        self.assertIsInstance(rh, management.RootHistory)

    def test__action(self):
        resp = mock.Mock()
        self.management.api.client.post = mock.Mock(
            return_value=(resp, 'body')
        )
        resp.status_code = 200
        self.management._action(1, 'body')
        self.assertEqual(1, self.management.api.client.post.call_count)
        resp.status_code = 400
        self.assertRaises(Exception, self.management._action, 1, 'body')
        self.assertEqual(2, self.management.api.client.post.call_count)

    def _mock_action(self):
        self.body_ = ""

        def side_effect_func(instance_id, body):
            self.body_ = body
        self.management._action = mock.Mock(side_effect=side_effect_func)

    def test_stop(self):
        self._mock_action()
        self.management.stop(1)
        self.assertEqual(1, self.management._action.call_count)
        self.assertEqual({'stop': {}}, self.body_)

    def test_reboot(self):
        self._mock_action()
        self.management.reboot(1)
        self.assertEqual(1, self.management._action.call_count)
        self.assertEqual({'reboot': {}}, self.body_)

    def test_migrate(self):
        self._mock_action()
        self.management.migrate(1)
        self.assertEqual(1, self.management._action.call_count)
        self.assertEqual({'migrate': {}}, self.body_)

    def test_migrate_to_host(self):
        hostname = 'hostname2'
        self._mock_action()
        self.management.migrate(1, host=hostname)
        self.assertEqual(1, self.management._action.call_count)
        self.assertEqual({'migrate': {'host': hostname}}, self.body_)

    def test_update(self):
        self._mock_action()
        self.management.update(1)
        self.assertEqual(1, self.management._action.call_count)
        self.assertEqual({'update': {}}, self.body_)

    def test_reset_task_status(self):
        self._mock_action()
        self.management.reset_task_status(1)
        self.assertEqual(1, self.management._action.call_count)
        self.assertEqual({'reset-task-status': {}}, self.body_)


class MgmtFlavorsTest(testtools.TestCase):

    def setUp(self):
        super(MgmtFlavorsTest, self).setUp()
        self.orig__init = management.MgmtFlavors.__init__
        management.MgmtFlavors.__init__ = mock.Mock(return_value=None)
        self.flavors = management.MgmtFlavors()
        self.flavors.api = mock.Mock()
        self.flavors.api.client = mock.Mock()
        self.flavors.resource_class = mock.Mock(return_value="flavor-1")
        self.orig_base_getid = base.getid
        base.getid = mock.Mock(return_value="flavor1")

    def tearDown(self):
        super(MgmtFlavorsTest, self).tearDown()
        management.MgmtFlavors.__init__ = self.orig__init
        base.getid = self.orig_base_getid

    def test_create(self):
        def side_effect_func(path, body, inst):
            return path, body, inst

        self.flavors._create = mock.Mock(side_effect=side_effect_func)
        p, b, i = self.flavors.create("test-name", 1024, 30, 2, 1)
        self.assertEqual("/mgmt/flavors", p)
        self.assertEqual("flavor", i)
        self.assertEqual("test-name", b["flavor"]["name"])
        self.assertEqual(1024, b["flavor"]["ram"])
        self.assertEqual(2, b["flavor"]["vcpu"])
        self.assertEqual(1, b["flavor"]["flavor_id"])


class MgmtDatastoreVersionsTest(testtools.TestCase):

    def setUp(self):
        super(MgmtDatastoreVersionsTest, self).setUp()
        self.orig__init = management.MgmtDatastoreVersions.__init__
        management.MgmtDatastoreVersions.__init__ = mock.Mock(
            return_value=None)
        self.ds_version = management.MgmtDatastoreVersions()
        self.ds_version.api = mock.Mock()
        self.ds_version.api.client = mock.Mock()
        self.ds_version.resource_class = mock.Mock(return_value="ds-version-1")
        self.orig_base_getid = base.getid
        base.getid = mock.Mock(return_value="ds-version1")

    def tearDown(self):
        super(MgmtDatastoreVersionsTest, self).tearDown()
        management.MgmtDatastoreVersions.__init__ = self.orig__init
        base.getid = self.orig_base_getid

    def _get_mock_method(self):
        self._resp = mock.Mock()
        self._body = None
        self._url = None

        def side_effect_func(url, body=None):
            self._body = body
            self._url = url
            return (self._resp, body)

        return mock.Mock(side_effect=side_effect_func)

    def test_create(self):
        def side_effect_func(path, body, *kw):
            return path, body

        self.ds_version._create = mock.Mock(side_effect=side_effect_func)
        p, b, = self.ds_version.create(
            "ds-version1", "mysql", "mysql", "image-id",
            ["mysql-server-5.5"], "true", "true")
        self.assertEqual("/mgmt/datastore-versions", p)
        self.assertEqual("ds-version1", b["version"]["name"])
        self.assertEqual("mysql", b["version"]["datastore_name"])
        self.assertEqual("mysql", b["version"]["datastore_manager"])
        self.assertEqual("image-id", b["version"]["image"])
        self.assertEqual(["mysql-server-5.5"], b["version"]["packages"])
        self.assertTrue(b["version"]["active"])
        self.assertTrue(b["version"]["default"])

    def test_get(self):
        def side_effect_func(path, ins):
            return path, ins
        self.ds_version._get = mock.Mock(side_effect=side_effect_func)
        p, i = self.ds_version.get('ds-version-1')
        self.assertEqual(('/mgmt/datastore-versions/ds-version-1', 'version'),
                         (p, i))

    def test_list(self):
        page_mock = mock.Mock()
        self.ds_version._paginated = page_mock
        self.ds_version.list()
        page_mock.assert_called_with('/mgmt/datastore-versions',
                                     'versions', None, None)
        self.ds_version.list(limit=10, marker="foo")
        page_mock.assert_called_with('/mgmt/datastore-versions',
                                     'versions', 10, "foo")

    def test_delete(self):
        resp = mock.Mock()
        resp.status_code = 202
        self.ds_version.api.client.delete = mock.Mock(
            return_value=(resp, None)
        )
        self.ds_version.delete('ds-version-1')
        self.assertEqual(1, self.ds_version.api.client.delete.call_count)
        self.ds_version.api.client.delete.assert_called_with(
            '/mgmt/datastore-versions/ds-version-1')

        resp.status_code = 400
        self.assertRaises(Exception, self.ds_version.delete, 'ds-version-2')
        self.assertEqual(2, self.ds_version.api.client.delete.call_count)
        self.ds_version.api.client.delete.assert_called_with(
            '/mgmt/datastore-versions/ds-version-2')

    def test_edit(self):
        self.ds_version.api.client.patch = self._get_mock_method()
        self._resp.status_code = 202
        self.ds_version.edit('ds-version-1', image="new-image-id")
        self.assertEqual('/mgmt/datastore-versions/ds-version-1', self._url)
        self.assertEqual({"image": "new-image-id"}, self._body)

        self._resp.status_code = 400
        self.assertRaises(Exception, self.ds_version.edit, 'ds-version-1',
                          "new-mgr", "non-existent-image")



# Copyright 2011 OpenStack Foundation
# Copyright 2013 Rackspace Hosting
# Copyright 2013 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools
from unittest import mock

from troveclient.v1 import security_groups

"""
Unit tests for security_groups.py
"""


class SecGroupTest(testtools.TestCase):

    def setUp(self):
        super(SecGroupTest, self).setUp()
        self.orig__init = security_groups.SecurityGroup.__init__
        security_groups.SecurityGroup.__init__ = mock.Mock(return_value=None)
        self.security_group = security_groups.SecurityGroup()
        self.security_groups = security_groups.SecurityGroups(1)

    def tearDown(self):
        super(SecGroupTest, self).tearDown()
        security_groups.SecurityGroup.__init__ = self.orig__init

    def test___repr__(self):
        self.security_group.name = "security_group-1"
        self.assertEqual('<SecurityGroup: security_group-1>',
                         self.security_group.__repr__())

    def test_list(self):
        sec_group_list = ['secgroup1', 'secgroup2']
        self.security_groups.list = mock.Mock(return_value=sec_group_list)
        self.assertEqual(sec_group_list, self.security_groups.list())

    def test_get(self):
        def side_effect_func(path, inst):
            return path, inst

        self.security_groups._get = mock.Mock(side_effect=side_effect_func)
        self.security_group.id = 1
        self.assertEqual(('/security-groups/1', 'security_group'),
                         self.security_groups.get(self.security_group))


class SecGroupRuleTest(testtools.TestCase):

    @mock.patch.object(security_groups.SecurityGroupRules, '__init__',
                       mock.Mock(return_value=None))
    @mock.patch.object(security_groups.SecurityGroupRule, '__init__',
                       mock.Mock(return_value=None))
    def setUp(self, *args):
        super(SecGroupRuleTest, self).setUp()
        self.security_group_rule = security_groups.SecurityGroupRule()
        self.security_group_rules = security_groups.SecurityGroupRules()

    def tearDown(self):
        super(SecGroupRuleTest, self).tearDown()

    def test___repr__(self):
        self.security_group_rule.group_id = 1
        self.security_group_rule.protocol = "tcp"
        self.security_group_rule.from_port = 80
        self.security_group_rule.to_port = 80
        self.security_group_rule.cidr = "0.0.0.0//0"
        representation = (
            "<SecurityGroupRule: "
            "( Security Group id: %d, "
            "Protocol: %s, From_Port: %d, "
            "To_Port: %d,  CIDR: %s )>"
            % (1, "tcp", 80, 80, "0.0.0.0//0")
        )

        self.assertEqual(representation,
                         self.security_group_rule.__repr__())

    def test_create(self):
        def side_effect_func(path, body, inst, return_raw=True):
            return path, body, inst

        self.security_group_rules._create = mock.Mock(
            side_effect=side_effect_func
        )
        p, b, i = self.security_group_rules.create(1, "0.0.0.0//0")
        self.assertEqual("/security-group-rules", p)
        self.assertEqual("security_group_rule", i)
        self.assertEqual(1, b["security_group_rule"]["group_id"])
        self.assertEqual("0.0.0.0//0", b["security_group_rule"]["cidr"])

    def test_delete(self):
        resp = mock.Mock()
        resp.status = 200
        body = None
        self.security_group_rules.api = mock.Mock()
        self.security_group_rules.api.client = mock.Mock()
        self.security_group_rules.api.client.delete = mock.Mock(
            return_value=(resp, body)
        )
        self.security_group_rules.delete(self.id)
        resp.status_code = 500
        self.assertRaises(Exception, self.security_group_rules.delete,
                          self.id)



# Copyright 2011 OpenStack Foundation
# Copyright 2013 Rackspace Hosting
# Copyright 2013 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from troveclient import base
from troveclient import common


class Quotas(base.ManagerWithFind):
    """Manage :class:`Quota` information."""

    resource_class = base.Resource

    def show(self, tenant_id):
        """Get a list of all quotas for a tenant id."""

        url = "/mgmt/quotas/%s" % tenant_id
        resp, body = self.api.client.get(url)
        common.check_for_exceptions(resp, body, url)
        if not body:
            raise Exception("Call to " + url + " did not return a body.")
        if 'quotas' not in body:
            raise Exception("Missing key value 'quotas' in response body.")
        return [self.resource_class(self, quota) for quota in body['quotas']]

    def update(self, id, quotas):
        """Set limits for quotas."""
        url = "/mgmt/quotas/%s" % id
        body = {"quotas": quotas}
        resp, body = self.api.client.put(url, body=body)
        common.check_for_exceptions(resp, body, url)
        if not body:
            raise Exception("Call to " + url + " did not return a body.")
        if 'quotas' not in body:
            raise Exception("Missing key value 'quotas' in response body.")
        return body['quotas']

    # Appease the abc gods
    def list(self):
        pass



#    Copyright 2012 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from troveclient.compat import exceptions


def get_authenticator_cls(cls_or_name):
    """Factory method to retrieve Authenticator class."""
    if isinstance(cls_or_name, type):
        return cls_or_name
    elif isinstance(cls_or_name, str):
        if cls_or_name == "keystone":
            return KeyStoneV3Authenticator
        elif cls_or_name == "auth1.1":
            return Auth1_1
        elif cls_or_name == "fake":
            return FakeAuth

    raise ValueError("Could not determine authenticator class from the given "
                     "value %r." % cls_or_name)


class Authenticator(object):
    """Helper class to perform Keystone or other miscellaneous authentication.

    The "authenticate" method returns a ServiceCatalog, which can be used
    to obtain a token.

    """

    URL_REQUIRED = True

    def __init__(self, client, type, url, username, password, tenant,
                 region=None, service_type=None, service_name=None,
                 service_url=None):
        self.client = client
        self.type = type
        self.url = url
        self.username = username
        self.password = password
        self.tenant = tenant
        self.region = region
        self.service_type = service_type
        self.service_name = service_name
        self.service_url = service_url

    def _authenticate(self, url, body, root_key='access'):
        """Authenticate and extract the service catalog."""
        # Make sure we follow redirects when trying to reach Keystone
        tmp_follow_all_redirects = self.client.follow_all_redirects
        self.client.follow_all_redirects = True

        try:
            resp, body = self.client._time_request(url, "POST", body=body)
        finally:
            self.client.follow_all_redirects = tmp_follow_all_redirects

        if resp.status == 201:  # Keystone v3
            try:
                token = resp.get('x-subject-token')
                return ServiceCatalog3(body, region=self.region,
                                       service_type=self.service_type,
                                       service_name=self.service_name,
                                       service_url=self.service_url,
                                       token=token)
            except exceptions.AmbiguousEndpoints:
                print("Found more than one valid endpoint. Use a more "
                      "restrictive filter")
                raise
            except KeyError:
                raise exceptions.AuthorizationFailure()
            except exceptions.EndpointNotFound:
                print("Could not find any suitable endpoint. Correct region?")
                raise
        elif resp.status == 200:  # Keystone pre-v3
            try:
                return ServiceCatalog(body, region=self.region,
                                      service_type=self.service_type,
                                      service_name=self.service_name,
                                      service_url=self.service_url,
                                      root_key=root_key)
            except exceptions.AmbiguousEndpoints:
                print("Found more than one valid endpoint. Use a more "
                      "restrictive filter")
                raise
            except KeyError:
                raise exceptions.AuthorizationFailure()
            except exceptions.EndpointNotFound:
                print("Could not find any suitable endpoint. Correct region?")
                raise

        elif resp.status == 305:
            return resp['location']
        else:
            raise exceptions.from_response(resp, body)

    def authenticate(self):
        raise NotImplementedError("Missing authenticate method.")


class KeyStoneV3Authenticator(Authenticator):
    def __init__(self, client, type, url, username, password, tenant,
                 region=None, service_type=None, service_name=None,
                 service_url=None):
        super(KeyStoneV3Authenticator, self).\
            __init__(client, type, url, username, password, tenant,
                     region=region, service_type=service_type,
                     service_name=service_name, service_url=service_url)

        # The Auth obect is needed because
        # troveclient.v1.instances.Instances._get_swift_client assumes
        # its v3 auth path is using a client which inherits from
        # keystone1.adapter.Adapter and then further goes after that
        # adapter's internal attributes to fetch them and send them
        # to the Swift client it creates.
        class Auth(object):
            def __init__(self, auth_url, username, password, project_name):
                token_str = "/auth/tokens"
                if auth_url.endswith(token_str):
                    auth_url = auth_url[:-len(token_str)]

                self.auth_url = auth_url
                self._username = username
                self._password = password
                self._project_name = project_name

        self.auth = Auth(url, username, password, tenant)

    def authenticate(self):
        if self.url is None:
            raise exceptions.AuthUrlNotGiven()
        return self._v3_auth(self.url)

    def _v3_auth(self, url):
        """Authenticate against a v3.0 auth service."""
        body = {'auth': {
                'identity': {
                    'methods': ['password'],
                    'password': {
                        'user': {
                            'domain': {'name': 'Default'},
                            'name': self.username,
                            'password': self.password
                        }
                    }
                }}}

        if self.tenant:
            body['auth']['scope'] = {'project': {
                                     'domain': {'name': 'Default'},
                                     'name': self.tenant}}

        return self._authenticate(url, body)


class KeyStoneV2Authenticator(Authenticator):
    def authenticate(self):
        if self.url is None:
            raise exceptions.AuthUrlNotGiven()
        return self._v2_auth(self.url)

    def _v2_auth(self, url):
        """Authenticate against a v2.0 auth service."""
        body = {"auth": {
            "passwordCredentials": {
                "username": self.username,
                "password": self.password}
        }
        }

        if self.tenant:
            body['auth']['tenantName'] = self.tenant

        return self._authenticate(url, body)


class Auth1_1(Authenticator):
    def authenticate(self):
        """Authenticate against a v2.0 auth service."""
        if self.url is None:
            raise exceptions.AuthUrlNotGiven()
        auth_url = self.url
        body = {
            "credentials": {
                "username": self.username,
                "key": self.password
            }}
        return self._authenticate(auth_url, body, root_key='auth')


class FakeAuth(Authenticator):
    """Useful for faking auth."""

    def authenticate(self):
        class FakeCatalog(object):
            def __init__(self, auth):
                self.auth = auth

            def get_public_url(self):
                return "%s/%s" % ('http://localhost:8779/v1.0',
                                  self.auth.tenant)

            def get_token(self):
                return self.auth.tenant

        return FakeCatalog(self)


class ServiceCatalog(object):
    """Represents a Keystone Service Catalog which describes a service.

    This class has methods to obtain a valid token as well as a public service
    url and a management url.

    """

    def __init__(self, resource_dict, region=None, service_type=None,
                 service_name=None, service_url=None, root_key='access'):
        self.catalog = resource_dict
        self.region = region
        self.service_type = service_type
        self.service_name = service_name
        self.service_url = service_url
        self.management_url = None
        self.public_url = None
        self.root_key = root_key
        self._load()

    def _load(self):
        if not self.service_url:
            self.public_url = self._url_for(attr='region',
                                            filter_value=self.region,
                                            endpoint_type="publicURL")
            self.management_url = self._url_for(attr='region',
                                                filter_value=self.region,
                                                endpoint_type="adminURL")
        else:
            self.public_url = self.service_url
            self.management_url = self.service_url

    def get_token(self):
        return self.catalog[self.root_key]['token']['id']

    def get_management_url(self):
        return self.management_url

    def get_public_url(self):
        return self.public_url

    def _url_for(self, attr=None, filter_value=None,
                 endpoint_type='publicURL'):
        """Fetch requested URL.

        Fetch the public URL from the Trove service for a particular
        endpoint attribute. If none given, return the first.
        """
        matching_endpoints = []
        if 'endpoints' in self.catalog:
            # We have a bastardized service catalog. Treat it special. :/
            for endpoint in self.catalog['endpoints']:
                if not filter_value or endpoint[attr] == filter_value:
                    matching_endpoints.append(endpoint)
            if not matching_endpoints:
                raise exceptions.EndpointNotFound()

        # We don't always get a service catalog back ...
        if 'serviceCatalog' not in self.catalog[self.root_key]:
            raise exceptions.EndpointNotFound()

        # Full catalog ...
        catalog = self.catalog[self.root_key]['serviceCatalog']

        for service in catalog:
            if service.get("type") != self.service_type:
                continue

            if (self.service_name and self.service_type == 'database' and
                    service.get('name') != self.service_name):
                continue

            endpoints = service['endpoints']
            for endpoint in endpoints:
                if not filter_value or endpoint.get(attr) == filter_value:
                    endpoint["serviceName"] = service.get("name")
                    matching_endpoints.append(endpoint)

        if not matching_endpoints:
            raise exceptions.EndpointNotFound()
        elif len(matching_endpoints) > 1:
            raise exceptions.AmbiguousEndpoints(endpoints=matching_endpoints)
        else:
            return matching_endpoints[0].get(endpoint_type, None)


class ServiceCatalog3(object):
    """Represents a Keystone Service Catalog which describes a service.

    This class has methods to obtain a valid token as well as a public service
    url and a management url.

    """

    def __init__(self, resource_dict, region=None, service_type=None,
                 service_name=None, service_url=None, token=None):
        self.body = resource_dict
        self.region = region
        self.service_type = service_type
        self.service_name = service_name
        self.service_url = service_url
        self.management_url = None
        self.public_url = None
        self.token = token
        self._load()

    def _load(self):
        if not self.service_url:
            self.public_url = self._url_for(attr='region',
                                            filter_value=self.region,
                                            endpoint_type="public")
            self.management_url = self._url_for(attr='region',
                                                filter_value=self.region,
                                                endpoint_type="admin")
        else:
            self.public_url = self.service_url
            self.management_url = self.service_url

    def get_token(self):
        return self.token

    def get_management_url(self):
        return self.management_url

    def get_public_url(self):
        return self.public_url

    def _url_for(self, attr=None, filter_value=None,
                 endpoint_type='public'):
        """Fetch requested URL.

        Fetch the public URL from the Trove service for a particular
        endpoint attribute. If none given, return the first.
        """
        """Fetch the requested end point URL.
         """
        matching_endpoints = []
        catalog = self.body['token']['catalog']
        for service in catalog:
            if service.get("type") != self.service_type:
                continue
            if (self.service_name and self.service_type == 'database' and
                    service.get('name') != self.service_name):
                continue
            endpoints = service['endpoints']
            for endpoint in endpoints:
                if endpoint.get('interface') == endpoint_type and \
                        (not filter_value or
                         endpoint.get(attr) == filter_value):
                    matching_endpoints.append(endpoint)
        if not matching_endpoints:
            raise exceptions.EndpointNotFound()
        else:
            return matching_endpoints[0].get('url')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Database v1 Configurations action implementations"""

import json

from osc_lib.command import command
from osc_lib import utils as osc_utils
from oslo_utils import uuidutils

from troveclient import exceptions
from troveclient.i18n import _


def set_attributes_for_print_detail(configuration):
    info = configuration._info.copy()
    info['values'] = json.dumps(configuration.values)
    del info['datastore_version_id']
    return info


class ListDatabaseConfigurations(command.Lister):

    _description = _("List database configurations")
    columns = ['ID', 'Name', 'Description', 'Datastore Name',
               'Datastore Version Name', 'Datastore Version Number']

    def get_parser(self, prog_name):
        parser = super(ListDatabaseConfigurations, self).get_parser(prog_name)
        parser.add_argument(
            '--limit',
            dest='limit',
            metavar='<limit>',
            type=int,
            default=None,
            help=_('Limit the number of results displayed.')
        )
        parser.add_argument(
            '--marker',
            dest='marker',
            metavar='<ID>',
            help=_('Begin displaying the results for IDs greater than the '
                   'specified marker. When used with --limit, set this to '
                   'the last ID displayed in the previous run.')
        )
        return parser

    def take_action(self, parsed_args):
        db_configurations = self.app.client_manager.database.configurations
        config = db_configurations.list(limit=parsed_args.limit,
                                        marker=parsed_args.marker)
        config = [osc_utils.get_item_properties(c, self.columns)
                  for c in config]
        return self.columns, config


class ShowDatabaseConfiguration(command.ShowOne):
    _description = _("Shows details of a database configuration group.")

    def get_parser(self, prog_name):
        parser = super(ShowDatabaseConfiguration, self).get_parser(prog_name)
        parser.add_argument(
            'configuration_group',
            metavar='<configuration_group>',
            help=_('ID or name of the configuration group'),
        )
        return parser

    def take_action(self, parsed_args):
        db_configurations = self.app.client_manager.database.configurations
        configuration = osc_utils.find_resource(
            db_configurations, parsed_args.configuration_group)
        configuration = set_attributes_for_print_detail(configuration)
        return zip(*sorted(configuration.items()))


class ListDatabaseConfigurationParameters(command.Lister):

    _description = _("Lists available parameters for a configuration group.")
    columns = ['Name', 'Type', 'Min Size', 'Max Size', 'Restart Required']

    def get_parser(self, prog_name):
        parser = super(ListDatabaseConfigurationParameters, self).\
            get_parser(prog_name)
        parser.add_argument(
            'datastore_version',
            metavar='<datastore_version>',
            help=_('Datastore version name or ID assigned to the '
                   'configuration group. ID is preferred if more than one '
                   'datastore versions have the same name.')
        )
        parser.add_argument(
            '--datastore',
            metavar='<datastore>',
            default=None,
            help=_('ID or name of the datastore to list configuration'
                   'parameters for. Optional if the ID of the'
                   'datastore_version is provided.')
        )
        return parser

    def take_action(self, parsed_args):
        db_configuration_parameters = self.app.client_manager.\
            database.configuration_parameters

        if uuidutils.is_uuid_like(parsed_args.datastore_version):
            params = db_configuration_parameters.parameters_by_version(
                parsed_args.datastore_version)
        elif parsed_args.datastore:
            params = db_configuration_parameters.parameters(
                parsed_args.datastore,
                parsed_args.datastore_version)
        else:
            raise exceptions.NoUniqueMatch(_('Either datastore version ID or '
                                             'datastore name needs to be '
                                             'specified.'))

        for param in params:
            setattr(param, 'min_size', getattr(param, 'min', '-'))
            setattr(param, 'max_size', getattr(param, 'max', '-'))
        params = [osc_utils.get_item_properties(p, self.columns)
                  for p in params]
        return self.columns, params


class ShowDatabaseConfigurationParameter(command.ShowOne):
    _description = _("Shows details of a database configuration parameter.")

    def get_parser(self, prog_name):
        parser = super(ShowDatabaseConfigurationParameter, self).\
            get_parser(prog_name)
        parser.add_argument(
            'datastore_version',
            metavar='<datastore_version>',
            help=_('Datastore version name or ID assigned to the '
                   'configuration group. ID is preferred if more than one '
                   'datastore versions have the same name.')
        )
        parser.add_argument(
            'parameter',
            metavar='<parameter>',
            help=_('Name of the configuration parameter.'),
        )
        parser.add_argument(
            '--datastore',
            metavar='<datastore>',
            default=None,
            help=_('ID or name of the datastore to list configuration'
                   ' parameters for. Optional if the ID of the'
                   ' datastore_version is provided.'),
        )
        return parser

    def take_action(self, parsed_args):
        db_configuration_parameters = self.app.client_manager.database. \
            configuration_parameters

        if uuidutils.is_uuid_like(parsed_args.datastore_version):
            param = db_configuration_parameters.get_parameter_by_version(
                parsed_args.datastore_version,
                parsed_args.parameter)
        elif parsed_args.datastore:
            param = db_configuration_parameters.get_parameter(
                parsed_args.datastore,
                parsed_args.datastore_version,
                parsed_args.parameter)
        else:
            raise exceptions.NoUniqueMatch(_('Either datastore version ID or '
                                             'datastore name needs to be '
                                             'specified.'))

        return zip(*sorted(param._info.items()))


class DeleteDatabaseConfiguration(command.Command):

    _description = _("Deletes a configuration group.")

    def get_parser(self, prog_name):
        parser = super(DeleteDatabaseConfiguration, self).get_parser(prog_name)
        parser.add_argument(
            'configuration_group',
            metavar='<configuration_group>',
            help=_('ID or name of the configuration group'),
        )
        return parser

    def take_action(self, parsed_args):
        db_configurations = self.app.client_manager.database.configurations
        try:
            configuration = osc_utils.find_resource(
                db_configurations, parsed_args.configuration_group)
            db_configurations.delete(configuration)
        except Exception as e:
            msg = (_("Failed to delete configuration %(c_group)s: %(e)s")
                   % {'c_group': parsed_args.configuration_group, 'e': e})
            raise exceptions.CommandError(msg)


class CreateDatabaseConfiguration(command.ShowOne):

    _description = _("Creates a configuration group.")

    def get_parser(self, prog_name):
        parser = super(CreateDatabaseConfiguration, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of the configuration group.'),
        )
        parser.add_argument(
            'values',
            metavar='<values>',
            help=_('Dictionary of the values to set.'),
        )
        parser.add_argument(
            '--datastore',
            metavar='<datastore>',
            default=None,
            help=_('Datastore assigned to the configuration group. Required '
                   'if default datastore is not configured.'),
        )
        parser.add_argument(
            '--datastore-version',
            metavar='<datastore_version>',
            default=None,
            help=_('Datastore version ID assigned to the '
                   'configuration group.'),
        )
        parser.add_argument(
            '--datastore-version-number',
            default=None,
            help=_('The version number for the database. The version number '
                   'is needed for the datastore versions with the same name.'),
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            default=None,
            help=_('An optional description for the configuration group.'),
        )
        return parser

    def take_action(self, parsed_args):
        db_configurations = self.app.client_manager.database.configurations
        config_grp = db_configurations.create(
            parsed_args.name,
            parsed_args.values,
            description=parsed_args.description,
            datastore=parsed_args.datastore,
            datastore_version=parsed_args.datastore_version,
            datastore_version_number=parsed_args.datastore_version_number)
        config_grp = set_attributes_for_print_detail(config_grp)
        return zip(*sorted(config_grp.items()))


class AttachDatabaseConfiguration(command.Command):

    _description = _("Attaches a configuration group to an instance.")

    def get_parser(self, prog_name):
        parser = super(AttachDatabaseConfiguration, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the instance'),
        )
        parser.add_argument(
            'configuration',
            metavar='<configuration>',
            type=str,
            help=_('ID or name of the configuration group to attach to the '
                   'instance.'),
        )
        return parser

    def take_action(self, parsed_args):
        manager = self.app.client_manager.database
        db_instances = manager.instances
        db_configurations = manager.configurations
        instance = osc_utils.find_resource(db_instances,
                                           parsed_args.instance)
        configuration = osc_utils.find_resource(
            db_configurations, parsed_args.configuration)
        db_instances.modify(instance, configuration)


class DetachDatabaseConfiguration(command.Command):

    _description = _("Detaches a configuration group from an instance.")

    def get_parser(self, prog_name):
        parser = super(DetachDatabaseConfiguration, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the instance.'),
        )
        return parser

    def take_action(self, parsed_args):
        db_instances = self.app.client_manager.database.instances
        instance = osc_utils.find_resource(db_instances,
                                           parsed_args.instance)
        db_instances.modify(instance)


class ListDatabaseConfigurationInstances(command.Lister):

    _description = _("Lists all instances associated "
                     "with a configuration group.")
    columns = ['ID', 'Name']

    def get_parser(self, prog_name):
        parser = super(ListDatabaseConfigurationInstances, self).\
            get_parser(prog_name)
        parser.add_argument(
            'configuration_group',
            metavar='<configuration_group>',
            help=_('ID or name of the configuration group.')
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            default=None,
            type=int,
            help=_('Limit the number of results displayed.')
        )
        parser.add_argument(
            '--marker',
            metavar='<ID>',
            default=None,
            type=str,
            help=_('Begin displaying the results for IDs greater than the '
                   'specified marker. When used with --limit, set this to '
                   'the last ID displayed in the previous run.')
        )
        return parser

    def take_action(self, parsed_args):
        db_configurations = self.app.client_manager.database.configurations
        configuration = osc_utils.find_resource(
            db_configurations, parsed_args.configuration_group)
        params = db_configurations.instances(configuration,
                                             limit=parsed_args.limit,
                                             marker=parsed_args.marker)
        instance = [osc_utils.get_item_properties(p, self.columns)
                    for p in params]
        return self.columns, instance


class DefaultDatabaseConfiguration(command.ShowOne):
    _description = _("Shows the default configuration of an instance.")

    def get_parser(self, prog_name):
        parser = super(DefaultDatabaseConfiguration, self).get_parser(
            prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the instance.'),
        )
        return parser

    def take_action(self, parsed_args):
        db_instances = self.app.client_manager.database.instances
        instance = osc_utils.find_resource(db_instances,
                                           parsed_args.instance)
        configs = db_instances.configuration(instance)
        return zip(*sorted(configs._info['configuration'].items()))


class SetDatabaseConfiguration(command.Command):
    _description = _("Change parameters for a configuration group.")

    def get_parser(self, prog_name):
        parser = super(SetDatabaseConfiguration, self).get_parser(prog_name)
        parser.add_argument(
            'configuration_group_id',
            help=_('Configuration group ID.'),
        )
        parser.add_argument(
            'values',
            metavar='<values>',
            help=_('Dictionary of the new values to set.'),
        )
        return parser

    def take_action(self, parsed_args):
        db_configurations = self.app.client_manager.database.configurations

        db_configurations.edit(
            parsed_args.configuration_group_id,
            parsed_args.values
        )


class UpdateDatabaseConfiguration(command.Command):
    _description = _("Update a configuration group.")

    def get_parser(self, prog_name):
        parser = super(UpdateDatabaseConfiguration, self).get_parser(prog_name)
        parser.add_argument(
            'configuration_group_id',
            help=_('Configuration group ID.'),
        )
        parser.add_argument(
            'values',
            metavar='<values>',
            help=_('Dictionary of the values to set.'),
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('New name of the configuration group.'),
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            default=None,
            help=_('An optional description for the configuration group.'),
        )
        return parser

    def take_action(self, parsed_args):
        db_configurations = self.app.client_manager.database.configurations

        db_configurations.update(
            parsed_args.configuration_group_id,
            parsed_args.values,
            name=parsed_args.name,
            description=parsed_args.description
        )



# Copyright (c) 2011 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from troveclient.compat.client import Dbaas   # noqa
from troveclient.compat.client import TroveHTTPClient     # noqa
from troveclient.compat.versions import Versions    # noqa
from troveclient.v1.accounts import Accounts   # noqa
from troveclient.v1.databases import Databases  # noqa
from troveclient.v1.diagnostics import DiagnosticsInterrogator    # noqa
from troveclient.v1.diagnostics import HwInfoInterrogator   # noqa
from troveclient.v1.flavors import Flavors   # noqa
from troveclient.v1.hosts import Hosts    # noqa
from troveclient.v1.instances import Instances  # noqa
from troveclient.v1.management import Management   # noqa
from troveclient.v1.management import MgmtFlavors  # noqa
from troveclient.v1.management import RootHistory  # noqa
from troveclient.v1.root import Root   # noqa
from troveclient.v1.storage import StorageInfo    # noqa
from troveclient.v1.users import Users   # noqa
